      PROGRAM AERSURFACE
C ======================================================================
C
C                  MAIN Module of the AERSURFACE Tool
C                        (Version dated 13016)
C
C                          January 16, 2013
C 
C           *** SEE AERSURFACE MODEL CHANGE BULLETIN MCB#1 ***
C
C     ON THE SUPPORT CENTER FOR REGULATORY AIR MODELS (SCRAM) WEBSITE
C
C=======================================================================
C
C       This revised version of AERSURFACE includes the following 
C       modifications relative to the previous version (dated 08009):
C
C-----  Bug Fixes:
C
C       1.  Corrected problems with reading GeoTIFF NLCD files that
C           are "tiled", rather than "stripped", as assumed in the 
C           original design.  Processing of "tiled" data with version
C           08009 could produce erroneous results.
C
C       2.  Incorporated a minimum distance from the land cover pixels
C           to the center of the study radius for purposes of applying an
C           inverse-distance weighting for estimating surface roughness.  
C           Sensitivity of roughness has been found in some cases with the 
C           pixel center located very close to the center of the study 
C           radius.  The minimum distance is set at 15 meters (one-half 
C           the 30m pixel resolution).
C
C       3.  Modified mod_AlbersProj.f to correct projection parameters
C           for the state of Washington, used in processing State NLCD
C           1992 files in a '.bin' format.
C
C       4.  Modified sub_GetLCCounts.f to correct the calculation of
C           the study radius as cells and corrected the calculation
C           of study center column and row indexes.  Use of incorrect
C           Fortran intrinsic function and mixed-mode computations
C           could have resulted in slight errors for these calculations.
C
C-----  Enhancements:
C
C       1.  Improved TIFF reader routine (Read_TiffTags) to read all
C           TIFF tags and GeoKeys.  Returns all values included in the
C           data file, and a separate routine (sub_Proc_TiffTags.cfm) 
C           evaluates a subset of tags and GeoKeys required to extract 
C           and process land cover data.  Provides greater flexibility 
C           to read NLCD data in GeoTIFF format from alternative sources.  
C           These enhancements also include more robust error handling
C           to avoid processing of GeoTIFF data that do not meet the
C           specifications required by AERSURFACE.
C
C       2.  Modified sub_GetUserInput routine to improve error handling
C           and reporting in order to minimize runtime errors and to 
C           facilitate identifying and correcting input errors associated 
C           with use of redirected input files.
C           
C-----  Miscellaneous:
C
C       1.  Modified surface roughness values for categories derived
C           as weighted averages of other categories (Categories 21, 
C           23, 43, and 91; Low Int Residential, Comm/Indust/Trans,
C           Mixed Forest, and Woody Wetlands) to reflect weighted 
C           geometric mean values rather than weighted arithmetic 
C           mean values. This change results in better consistency 
C           between the method for deriving roughness estimates based 
C           on a weighted mix of other categories and the method used 
C           to process the roughness values across the domain within 
C           AERSURFACE. 
C
C           Category weights used in determining the weighted mean
C           values were also adjusted as part of this reassessment of
C           derived roughness categories.  The weights assigned for the 
C           weighted geometric mean values are more consistent with
C           the definitions of the categories used in determining
C           the weighted means.
C
C       2.  Updated subroutines UTMGEO and NADCON to be consistent 
C           with the updated AERMAP Terrain Processor, dated 09040.
C           Subroutine UTMGEO was also moved from aersurface.f to a
C           separate file (sub_UTMGEO.f).
C
C       3.  Modified sub_GetLCCounts.f for mixed-mode computations
C           involving INTEGER (KIND=8) variables and constants.
C
C 
C-----  MODIFIED BY:    Clint R. Tillerson
C                       AMEC Environment & Infrastructure
C
C                       Roger W. Brode
C                       U.S. EPA, OAQPS/AQAD
C                       Air Quality Modeling Group
C
C                       January 16, 2013
C
C-----  MODIFIED FROM:        AERSURFACE
C                       (Version Dated 08009)
C
C=======================================================================
C
C     Purpose:  To objectively assess U.S. Geological Survey (USGS) 
C               National Land Cover Data (NLCD)and determine the surface 
C               characteristics (surface roughness length, Bowen ratio, 
C               and albedo) for purposes of processing meteorological
C               data for use with AERMOD.  AERSURFACE produces an 
C               output file formatted for Stage 3 processing through the 
C               AERMET meteorological processor for a user-specified 
C               location.  The current version supports the 1992 NLCD 
C               archives (NLCD92).
C
C        PROGRAMMED BY: Clint R. Tillerson
C                       James O. Paumier
C                       MACTEC Federal Programs, Inc.
C                       Research Triangle Park, North Carolina  27709
C
C                   and
C
C                       Roger W. Brode
C                       U.S. EPA, OAQPS/AQAD
C                       Air Quality Modeling Group
C                       Research Triangle Park, North Carolina  27711
C     
C        DATE:    January 9, 2008
C     
C ======================================================================

      USE AlbersProjectionData
      USE LandCoverParams
      USE MyConstants
      USE UserParams
      USE ControlParams
      USE AvgParams

      IMPLICIT NONE

C---- Initialize data
      run_start(:)=0
      CALL DATE_AND_TIME (big_ben(1), big_ben(2), big_ben(3), run_start)
      
      CALL InitStateInfo (STATEINFO)
      
      CALL InitLCSurfChar

C***********************************************************************
C     The control data are prompt-driven, interactive input.
C     A file with the same information can be used for redirected input
C***********************************************************************

      CALL GetUserInput

C***********************************************************************
C     Initialize data control parameters
C     STATE files - hard coded in subroutine
C     SEAMLESS files - extracted from GeoTIFF file via a subroutine
C***********************************************************************    

      CALL InitControlParams 

C-----------------------------------------------------------------------
C    Get counts of land use classes by sector
C-----------------------------------------------------------------------

      CALL GetLCCounts

C-----------------------------------------------------------------------
C    Compute the sector averaged surface characteristics - 
C    Inverse Distance or Simple method - set in mod_AvgParams.f
C    6/28/07, CRT: Simple for Albedo, distance weighted geometric mean
C       for roughness, straight geometric mean for Bowen ratio    
C-----------------------------------------------------------------------

C     Use simple (arithmetic) area weighted average for albedo
      CALL AvgSfcCharsSimple

C     Use geometric mean for Bowen (area weighted) and Zo (inv. distance weighted)
      CALL AvgSfcCharsGeom

C-----------------------------------------------------------------------
C    Write the results to the AERMET-format output file for Stage 3
C    Write the results plus additional information to the log file
C-----------------------------------------------------------------------

      CALL WriteSfcChars

      STOP
      END

C***********************************************************************
C     SUBROUTINE STRAIGHTSORT
C 
C     Purpose: To sort an array of integer values into ascending order
C              by straight insertion (not efficient, but is fine for
C              small N)
C
C     Assumptions: If 360 is enterd as a starting direction, it will
C                  be converted to to 0
C
C     I/O:  NSect = Number of wind direction sectors entered by the user
C           RealArray() = Array of wind sectors
C
C     Called By: main program
C
C     Calls To:  <none>
C
C***********************************************************************
      SUBROUTINE STRAIGHTSORT(NSect, IntArray)

      IMPLICIT NONE

      INTEGER :: NSect, i,j, TempVal
      INTEGER :: IntArray(NSect)

	DO j = 1,NSect
	   IF( IntArray(j) .EQ. 360 )THEN
             IntArray(j) = 0
	       EXIT
	   ENDIF
      END DO

      DO j = 2, NSect
         TempVal = IntArray(j)
         DO i = j-1, 1, -1
            IF (IntArray(i) .LE. TempVal ) GO TO 100
            IntArray(i+1) = IntArray(i)
         ENDDO
         i = 0
  100    IntArray(i+1) = TempVal
      ENDDO

      RETURN
      END



C***********************************************************************
C    SUBROUTINE LATLON2ALBERS
C
C    Purpose: To convert latutude and longitude expressed in decimal
C             degrees to x,y coordinates using Albers Equal Area Conic
C             projection
C
C    Assumptions:  All States use the same projection parameters
C                  (see Modules.for for the parameters)
C
C    Input:  CLat = Latitude (decimal degrees)
C            CLon = Longitude (decimal degrees)
C
C    Output: CAlbersX = Albers projection X-coordinate
C            CAlbersY = Albers projection Y-coordinate
C
C    Reference: Map Projections - A Working Manual, John P. Snyder,
C               Geological Survey Professional Paper 1395, 1987
C
C***********************************************************************
      SUBROUTINE LATLON2ALBERS(CLat, CLon, CAlbersX, CAlbersY)
      USE AlbersProjectionData
      USE MyConstants

      IMPLICIT NONE
      DOUBLE PRECISION :: CLat, CLon, CAlbersX, CAlbersY, Xlat
      DOUBLE PRECISION :: ComputeQ
      DOUBLE PRECISION :: Q, Q0, Q1, Q2, M1, M2, XN, C, Rho0, Theta, Rho


C---- Intermediate calculation: Q
      Xlat = Deg2Rad * CLat            ! latitude of site in radians
      Q = ComputeQ(Xlat)

C---- Intermediate calculation: Q0
      Xlat = Deg2Rad * ProjOrigin      ! projection origin in radians
      Q0 = ComputeQ(Xlat)

C---- Intermediate calculations: Q1, M1
      Xlat = Deg2Rad * StdParallel1    ! 1st std parallel in radians
      Q1 = ComputeQ(Xlat)
      M1 = dcos(Xlat) / dsqrt(1.0D0 - Esquared * dsin(XLat)*dsin(XLat) )

C---- Intermediate calculations: Q2, M2
      Xlat = Deg2Rad * StdParallel2    ! 2nd std parallel in radians
      Q2 = ComputeQ(Xlat)
      M2 = dcos(Xlat) / dsqrt(1.0D0 - Esquared * dsin(XLat)*dsin(XLat) )

C---- Intermediate calculation: N (variable name = XN)
      XN = (M1 * M1 - M2 * M2) / (Q2 - Q1)

C---- Intermediate calculation: C
      C = M1 * M1 + XN * Q1

C---- Intermediate calculation: Theta (in degrees)
      Theta = XN * (CLon - CentMeridian)

C---- Intermediate calculation: Rho0
      Rho0 = EarthRadius * dsqrt(C - XN * Q0) / XN

C---- Intermediate calculation: Rho
      Rho = EarthRadius * dsqrt(C - XN * Q) / XN

C---- Final calculation - x,y returned to calling program
      CAlbersX = Rho * dsin(Deg2Rad * Theta)
      CAlbersY = Rho0 - Rho * dcos(Deg2Rad * Theta)

      RETURN
      END



C***********************************************************************
C    SUBROUTINE ALBERS2LATLON
C
C    Purpose: To convert latutude and longitude expressed in decimal
C             degrees to x,y coordinates using Albers Equal Area Conic
C             projection
C
C    Assumptions:  All Statesa use the same projection parameters
C                  (see Modules.for for the parameters)
C
C    Input:  CAlbersX = Albers projection X-coordinate
C            CAlbersY = Albers projection Y-coordinate
C
C    Output: XLat = Latitude (decimal degrees)
C            XLon = Longitude (decimal degrees)
C
C    Reference: Map Projections - A Working Manual, John P. Snyder,
C               Geological Survey Professional Paper 1395, 1987
C               See page 102 for equations used below
C
C***********************************************************************
      SUBROUTINE ALBERS2LATLON(CAlbersX, CAlbersY, XLat, XLon)
      USE AlbersProjectionData
      USE MyConstants
      IMPLICIT NONE

      DOUBLE PRECISION :: ComputeQ
      DOUBLE PRECISION :: Beta, temp
      DOUBLE PRECISION :: CAlbersX, CAlbersY, Xlat, Xlon
      DOUBLE PRECISION :: Q, Q0, Q1, Q2, M1, M2, XN, C, Rho0, Theta, Rho


C---- Q0, Q1, Q2, M1, M2, XN, C, and Rho are as in LatLon2Albers

C---- Intermediate calculation: Q0
      Xlat = Deg2Rad * ProjOrigin      ! projection origin in radians
      Q0 = ComputeQ(Xlat)

C---- Intermediate calculations: Q1, M1
      Xlat = Deg2Rad * StdParallel1    ! 1st std parallel in radians
      Q1 = ComputeQ(Xlat)
      M1 = dcos(Xlat) / dsqrt(1.0D0 - Esquared * dsin(XLat)*dsin(XLat) )

C---- Intermediate calculations: Q2, M2
      Xlat = Deg2Rad * StdParallel2    ! 2nd std parallel in radians
      Q2 = ComputeQ(Xlat)
      M2 = dcos(Xlat) / dsqrt(1.0D0 - Esquared * dsin(XLat)*dsin(XLat) )

C---- Intermediate calculation: N (variable name = XN)
      XN = (M1 * M1 - M2 * M2) / (Q2 - Q1)

C---- Intermediate calculation: C
      C = M1 * M1 + XN * Q1

C---- Intermediate calculation: Rho0
      Rho0 = EarthRadius * dsqrt(C - XN * Q0) / XN

C---- Intermediate calculation: Rho
      Rho = dsqrt(CAlbersX**2 + (Rho0 - CAlbersY)**2)

C---- Intermediate calculation: Q
      Q = (C - (Rho**2) * (XN**2) / (EarthRadius**2)) / XN

C---- Intermediate calculation: Theta
      Theta = DATAN(CAlbersX / (Rho0 - CAlbersY)) * Rad2Deg

C     Compute the denominator
      Temp = 1.0D0-((1.0D0-Esquared)/(2.0D0*dsqrt(Esquared))) * 
     *       DLOG((1.0D0-dsqrt(Esquared))/(1.0D0+dsqrt(Esquared)))

C---- Beta is the authalic latitude to be used in the computations of
C     latitude by the series method
      Beta = DASIN(Q/Temp)

C---- Final calculation - latitude, longitude returned to calling program
C     We are using the series solution rather than the iterative
C     method to get the latitude
      XLon = CentMeridian + Theta / XN
      XLat = Beta + (1.0D0/3.0D0 * Esquared +
     &        31.0D0/180.0D0 * Esquared**2 +
     &        517.0D0/5040.0D0 * Esquared**3) * dsin(2.0D0*Beta) +
     &        (23.0D0/360.0D0 * Esquared**2 + 
     &        251.0D0/3780.0D0 * Esquared**3) * dsin(4.0D0*Beta) +
     &        (761.0D0/45360.0D0 * Esquared**3) * dsin(6.0D0*Beta)

      XLat= XLat * Rad2Deg

      RETURN
      END


C***********************************************************************
C    SUBROUTINE CONFIRMBOUNDS
C
C    Purpose: To check that the study area is within the boundaries of
C             the NLCD file
C
C     Called By: main program
C
C     Calls To:  <none>
C
C***********************************************************************
      SUBROUTINE CONFIRMBOUNDS(RadiusAsCells, CenterCol, CenterRow,
     &                         StateCols, StateRows, OK2Proceed)
      
      IMPLICIT NONE

      INTEGER (KIND=8) :: CenterCol, CenterRow, StateCols, StateRows
      INTEGER (KIND=8) :: RadiusAsCells
      LOGICAL :: OK2Proceed

      OK2Proceed = .true.

      IF( CenterCol - RadiusAsCells .LT. 0 )THEN
         OK2Proceed = .false.
         PRINT *, ' Study area too close to western edge of state'
         PRINT *, ' Study center column = ', CenterCol
      ENDIF

      IF( CenterCol + RadiusAsCells .GT. StateCols )THEN
         OK2Proceed = .false.
         PRINT *, ' Study area too close to eastern edge of state'
         PRINT *, ' Study center column = ', CenterCol
      ENDIF

      IF( CenterRow - RadiusAsCells .LT. 0 )THEN
         OK2Proceed = .false.
         PRINT *, ' Study area too close to southern edge of state'
         PRINT *, ' Study center row = ', CenterRow
      ENDIF

      IF( CenterRow + RadiusAsCells .GT. StateRows )THEN
         OK2Proceed = .false.
         PRINT *, ' Study area too close to northern edge of state'
         PRINT *, ' Study center row = ', CenterRow
      ENDIF

      RETURN
      END


C***********************************************************************
C     FUNCTION ComputeQ
C
C     Purpose: To compute the Q term in the lat/lon conversion to Albers
C
C     Called By: LatLon2Albers, Albers2LatLon
C
C     Calls To:  <none>
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION ComputeQ(Xlat)
      USE AlbersProjectionData

      IMPLICIT NONE

      DOUBLE PRECISION :: Xlat, Term1, Term2
      
      Term1 = dsin(XLat) / (1.0D0 - (Esquared * dsin(XLat)*dsin(XLat)))
      Term2 = (1.0D0 / (2.0D0 * dsqrt(Esquared))) *
     &        dlog((1.0D0 - dsqrt(Esquared) * dsin(Xlat)) /
     &            (1.0D0 + dsqrt(Esquared) * dsin(Xlat)) )
      ComputeQ = (1.0D0 - Esquared) * (Term1 - Term2)

      RETURN
      END


C***********************************************************************
C     LWRUPR Module of the AMS/EPA Regulatory Model - AERMOD
C
C     PURPOSE: Transfer All Characters From Lower Case To
C              Upper Case (Using INDEX Intrinsic Function)
C
C     PROGRAMMER: Roger Brode, Kevin Stroupe
C     MODIFIED BY: Jim Paumier for AERSURFACE
C
C     DATE:    March 2, 1992
C              May 16, 2006
C
C     INPUTS:  String to convert
C
C     OUTPUTS: Converted string, all uppercase
C
C     CALLED FROM:   main program
C***********************************************************************
      SUBROUTINE LWRUPR(String2Convert)
C
C     Variable Declarations
      IMPLICIT NONE
      CHARACTER (LEN=*) :: String2Convert

      INTEGER :: I, INDCHK, ISTRG
      CHARACTER (LEN=26) :: UPCASE, LWCASE

C     Variable Initializations
      DATA UPCASE/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      DATA LWCASE/'abcdefghijklmnopqrstuvwxyz'/

      ISTRG = LEN(String2Convert)
      DO I = 1, ISTRG
         IF (String2Convert(I:I) .NE. ' ') THEN
            INDCHK = INDEX(LWCASE,String2Convert(I:I))
            IF (INDCHK .NE. 0) THEN
               String2Convert(I:I) = UPCASE(INDCHK:INDCHK)
            END IF
         END IF
      END DO

      RETURN
      END
