
C=======================================================================
      MODULE AlbersProjectionData
C=======================================================================

C     Ellipsoid data

C       Esquared:     Square of eccentricity
C       EarthRadius:  Radius of the earth GRS 80 expressed in meters

C     Albers Projection Information for the USGS NLCD State data

C       CentMeridian: Central Meridian (lambda0 in equations in USGS
C                     reference - see header in Subr. LatLon2Albers
C                     for complete reference)

C       ProjOrigin:   Projection origin
C       StdParallel1: First standard parallel
C       StdParallel2: Second standard parallel
C ----------------------------------------------------------------------

      IMPLICIT NONE
      SAVE

c      DOUBLE PRECISION, PARAMETER :: Esquared = 0.006694380D0
c      DOUBLE PRECISION, PARAMETER :: EarthRadius = 6378137.0D0


c      DOUBLE PRECISION, PARAMETER :: CentMeridian = -96.0D0
c      DOUBLE PRECISION, PARAMETER :: ProjOrigin   = 23.0D0
c      DOUBLE PRECISION, PARAMETER :: StdParallel1 = 29.5D0
c      DOUBLE PRECISION, PARAMETER :: StdParallel2 = 45.5D0

      CHARACTER*10     :: LCDatum     ! Horizontal datum of land cover file

      DOUBLE PRECISION :: Esquared    ! Eccentricity^2
      DOUBLE PRECISION :: EarthRadius ! Radius of the earth
      
      DOUBLE PRECISION :: CentMeridian  ! (Central Meridian) Center Longitude
      DOUBLE PRECISION :: ProjOrigin    ! Origin Latitude
      
      DOUBLE PRECISION :: StdParallel1  ! Southernmost Latitude
      DOUBLE PRECISION :: StdParallel2  ! Northernmost Latitude
      
      ! Spheroid for call to UTMGEO for conversion of UTM to Lat/Lon
      ! Based on datum entered by user NAD27 or NAD83
      ! NAD27 => Clark 1866 = 0; NAD83 => GRS80 = 4
      INTEGER          :: iSphere
      

C=======================================================================
C     Define a data type to hold the information that defines the
C     location of the State in an Albers equal area projection
C
C     Note that the State NLCD data from USGS forms a rectangle and
C     that there is a great deal of 'black' space outside the actual
C     State borders with 0 as the land cover value.
C
C     NAreas:          Number of distinct States + divisions
C                      within a State

C     StatePostalCode: 2-character state abbreviation

C     NumCols:         Number of columns in the State NLCD data
C     NumRows:         Number of rows in the State NLCD data

C     ULX:             x-coordinate (Albers Easting) of the upper left corner
C                      of the State NLCD data
C     ULY:             y-coordinate (Albers Northing) of the upper left corner
C                      of the State NLCD data
C     LRX:             x-coordinate (Albers Easting) of the lower right corner
C                      of the State NLCD data
C     LRY:             y-coordinate (Albers Northing) of the lower right corner
C                      of the State NLCD data

C     STATEINPUT:      Derived variable type that defines the
C                      location of the State in an Albers equal area
C                      projection
C-----------------------------------------------------------------------

      INTEGER, PARAMETER :: NAreas = 56

      TYPE STATEINPUT
            CHARACTER (LEN=2) :: StatePostalCode
            INTEGER (kind=8)  :: NumCols, NumRows
            INTEGER (kind=8)  :: ULX, ULY, LRX, LRY
      END TYPE STATEINPUT
      
      TYPE (STATEINPUT), DIMENSION (NAreas) :: STATEINFO

C---- In the data below, (used for State binary and GeoTIFF 1992 files):
C       1) data are not available for either Alaska or Hawaii
C       2) California and Texas are divided into multiple areas
C       3) to denote the presence of a state postal code but no data,
C          the rows and columns are set to 1,1 so a check in the main
C          program can determine if the user entered a valid postal code

      CONTAINS

	   SUBROUTINE InitStateInfo (STATEINFO)

         TYPE (STATEINPUT), DIMENSION (NAreas) :: STATEINFO

C----    Data are not available for Alaska
         STATEINFO(1) = STATEINPUT('AK',
     &                 1,1,0,0,0,0)
         STATEINFO(2) = STATEINPUT('AL',
     &                 11864,19175,696270,1386300,1052160,811080)
         STATEINFO(3) = STATEINPUT('AZ',
     &                 20036,23328,-1747230,1701780,-1146180,1001970)
         STATEINFO(4) = STATEINPUT('AR',
     &                 14869,13461,122340,1510800,568380,1107000)

C----    California is in 2 sections: N, S - see array elements 51 and 52
         STATEINFO(5) = STATEINPUT('CA',
     &                 1,1,0,0,0,0)
         STATEINFO(6) = STATEINPUT('CO',
     &                 21418,16915,-1146810,2074020,-504300,1566600)
         STATEINFO(7) = STATEINPUT('CT',
     &                 5248,5264,1831260,2368230,1988670,2210340)
         STATEINFO(8) = STATEINPUT('DE',
     &                 3300,5170,1703490,2055690,1802460,1900620)
         STATEINFO(9) = STATEINPUT('FL',
     &                 27314,23393,783810,969660,1603200,267900)
         STATEINFO(10) = STATEINPUT('GA',
     &                 17074,17648,926910,1417050,1439100,887640)

C----    Data are not available for Hawaii

         STATEINFO(11) = STATEINPUT('HI',
     &                 1,1,0,0,0,0)
         STATEINFO(12) = STATEINPUT('ID',
     &                 17567,28387,-1715970,3060150,-1188990,2208570)
         STATEINFO(13) = STATEINPUT('IL',
     &                 11835,21096,378030,2201310,733050,1568460)
         STATEINFO(14) = STATEINPUT('IN',
     &                 9184,15757,686850,2139030,962340,1666350)
         STATEINFO(15) = STATEINPUT('IA',
     &                 17834,11712,-52620,2288880,482370,1937550)
         STATEINFO(16) = STATEINPUT('KS',
     &                 21829,11788,-532680,1903920,122160,1550310)
         STATEINFO(17) = STATEINPUT('KY',
     &                 21849,11189,569070,1849050,1224510,1513410)
         STATEINFO(18) = STATEINPUT('LA',
     &                 17234,14956,181380,1115490,698370,666840)
         STATEINFO(19) = STATEINPUT('ME',
     &                 11408,17729,1923780,3016020,2265990,2484180)
         STATEINFO(20) = STATEINPUT('MD',
     &                 13575,7014,1396050,2038290,1803270,1827900)

         STATEINFO(21) = STATEINPUT('MA',
     &                 10672,6312,1826460,2483430,2146590,2294100)
         STATEINFO(22) = STATEINPUT('MI',
     &                 22795,23944,428490,2835390,1112310,2117100)
         STATEINFO(23) = STATEINPUT('MN',
     &                 19430,21780,-92400,2931270,490470,2277900)
         STATEINFO(24) = STATEINPUT('MS',
     &                 11058,18639,407970,1361550,739680,802410)
         STATEINFO(25) = STATEINPUT('MO',
     &                 19665,17045,18600,1964100,608520,1452780)
         STATEINFO(26) = STATEINPUT('MT',
     &                 30100,19094,-1498260,3044730,-595290,2471940)
         STATEINFO(27) = STATEINPUT('NE',
     &                 24363,12153,-671730,2250990,59130,1886430)
         STATEINFO(28) = STATEINPUT('NV',
     &                 18756,28903,-2037840,2358600,-1475190,1491540)
         STATEINFO(29) = STATEINPUT('NH',
     &                 5364,10237,1879380,2734830,2040270,2427750)
         STATEINFO(30) = STATEINPUT('NJ',
     &                 4297,9932,1721790,2242860,1850670,1944930)

         STATEINFO(31) = STATEINPUT('NM',
     &                 20583,21277,-1233900,1629780,-616440,991500)
         STATEINFO(32) = STATEINPUT('NY',
     &                 23005,17455,1317210,2663820,2007330,2140200)
         STATEINFO(33) = STATEINPUT('NC',
     &                 26378,12094,1047690,1699530,1839000,1336740)
         STATEINFO(34) = STATEINPUT('ND',
     &                 19386,12143,-624210,2913750,-42660,2549490)
         STATEINFO(35) = STATEINPUT('OH',
     &                 12371,15385,922170,2251080,1293270,1789560)
         STATEINFO(36) = STATEINPUT('OK',
     &                 25428,13368,-621210,1573800,141600,1172790)
         STATEINFO(37) = STATEINPUT('OR',
     &                 23924,20221,-2301360,2907510,-1583670,2300910)
         STATEINFO(38) = STATEINPUT('PA',
     &                 17370,11105,1261380,2295150,1782450,1962030)
         STATEINFO(39) = STATEINPUT('RI',
     &                 2463,3507,1968930,2374110,2042790,2268930)
         STATEINFO(40) = STATEINPUT('SC',
     &                 15620,11915,1138290,1458540,1606860,1101120)

         STATEINFO(41) = STATEINPUT('SD',
     &                 20585,13774,-652560,2577510,-35040,2164320)
         STATEINFO(42) = STATEINPUT('TN',
     &                 26006,9396,498660,1613760,1278810,1331910)

C----    Texas is in 3 sections: N, SE, SW - see array elements 54-56
         STATEINFO(43) = STATEINPUT('TX',
     &                 1,1,0,0,0,0)
         STATEINFO(44) = STATEINPUT('UT',
     &                 16563,20729,-1582080,2250990,-1085220,1629150)
         STATEINFO(45) = STATEINPUT('VT',
     &                 5659,10140,1758930,2707020,1928670,2402850)
         STATEINFO(46) = STATEINPUT('VA',
     &                 23630,13032,1088700,1967100,1797570,1576170)
         STATEINFO(47) = STATEINPUT('WA', 
     &                 19975,14795,-2144460,3177720,-1545240,2733900)
         STATEINFO(48) = STATEINPUT('WV',
     &                 13265,13240,1154670,2064600,1552590,1667430)
         STATEINFO(49) = STATEINPUT('WI',
     &                 17414,18513,241980,2715090,764370,2159730)
         STATEINFO(50) = STATEINPUT('WY',
     &                  20575,17088,-1250670,2539890,-633450,2027280)

         STATEINFO(51) = STATEINPUT('DC',
     &                  662,784,1610190,1936680,1630020,1913190)
         STATEINFO(52) = STATEINPUT('CN',
     &                  14971,20260,-2361900,2455830,-1912800,1848060)  ! Calif North
         STATEINFO(53) = STATEINPUT('CS',
     &                  23851,20260,-2361900,1849830,-1646400,1242060)  ! Calif South
         STATEINFO(54) = STATEINPUT('NT',
     &                  28285,16717,-664320,1518240,184200,1016760)   ! Northern Texas
         STATEINFO(55) = STATEINPUT('WT',
     &                  22199,20435,-1000650,1047900,-334710,434880)  ! SW Texas
         STATEINFO(56) = STATEINPUT('ET',
     &                  20078,24577,-364860,1047900,237450,310620)    ! SE Texas

         END SUBROUTINE InitStateInfo

      END MODULE AlbersProjectionData