
      SUBROUTINE GetLCCounts
      
C ----------------------------------------------------------------------
C     
C     Purpose:  Get Land Cover counts for user-specified domain for 
C               albedo, Bowen ratio and surface roughness.
C
C     NOTE: The term cell refers to the area that one pixel in
C           the NLCD covers

C ----------------------------------------------------------------------


      USE AlbersProjectionData
      USE LandCoverParams
      USE MyConstants
      USE UserParams
      USE FileUnits
      USE ControlParams
      USE AvgParams

      IMPLICIT NONE
      
      INTEGER :: MissCountBa, MissCountZ0   ! Total missing data cells + cells outside of state boundary
      
      INTEGER (KIND=1)   :: TempLC(4)  ! 1-byte land cover variable read from file
      INTEGER (KIND=2)   :: tmpInt2

      DOUBLE PRECISION :: CellX, CellY      ! Cell albers coordinates (in meters)
      DOUBLE PRECISION :: CLat,CLon  ! DEBUG
      DOUBLE PRECISION :: XPTIN, YPTIN, XPTOUT, YPTOUT  ! for NADCON
      DOUBLE PRECISION :: DLOS, DLAS, DLOM, DLAM  ! for NADCON
      DOUBLE PRECISION :: BaAlbersLLX, BaAlbersLLY ! lower left albers coordinates for bowen and albedo grid
      DOUBLE PRECISION :: ZoAlbersLLX, ZoAlbersLLY ! lower left albers coordinates for Zo grid
      
      INTEGER (KIND=8) :: CellLC  ! land cover category from cell

      INTEGER (KIND=8) :: icol, jrow, kcol, idos  ! double precision counters
      INTEGER (KIND=4) :: isector                 ! sector counter
      INTEGER (KIND=4) :: kdum
      
      INTEGER (kind=8) :: tmprow, tmpcol 
 
      INTEGER (KIND=4) :: Key     ! key for datum shift
      LOGICAL :: NADShft ! Flag to indicate if NAD shift is necessary
      
      INTEGER (kind=8)   :: thisrec        ! current record 

      integer (kind=8)   :: tilesAcross    ! width of image in tiles
      integer (kind=8)   :: tilesDown      ! length (height) of image in tiles
      integer (kind=8)   :: tileNum        ! number of current tile read
      integer (kind=8)   :: tileCol        ! tile position (columns of tiles)
      integer (kind=8)   :: tileRow        ! tile position (rows of tiles)
      integer (kind=8)   :: tileFirstRow   ! first row in current tile
      integer (kind=8)   :: tileFirstCol   ! first col in current tile
      
      INTEGER (KIND=4)   :: DummyZone      ! Dummy Zone variable for UTMGEO call 

      LOGICAL :: OK2Proceed ! OK flag
      LOGICAL :: NODATA     ! flag for existence of NAD Grid files


C --- Initialize logical flags     
      byteswap = .false.
      NODATA = .false.    
      NADShft = .false.
      LC1992 = .true.
      
C --- Initialize arrays      
      LandCovTotBr  = 0        
      LandCovTotAlb = 0      
      LandCovTotZ0  = 0
   

C --- determine which study radius is longer and set as max
      StudyRadiusMax = MAX(StudyRadiusBa, StudyRadiusZ0)
      
C --- convert study radii to meters
      StudyRadiusBaMeters = StudyRadiusBa*1000.0D0
      StudyRadiusZ0Meters = StudyRadiusZ0*1000.0D0
      StudyRadiusMaxMeters = StudyRadiusMax*1000.0D0
      
C --- convert study radii to cell lengths
      StudyRadiusBaAsCells  = IDNINT(StudyRadiusBa*1000.0D0/CellRes)
      StudyRadiusZ0AsCells  = IDNINT(StudyRadiusZ0*1000.0D0/CellRes)
      StudyRadiusMaxAsCells = IDNINT(StudyRadiusMax*1000.0D0/CellRes)

C-----------------------------------------------------------------------
C     If the user entered UTM coordinates, convert to lat/lon
C-----------------------------------------------------------------------

      IF( CoordType == 'UTM' )THEN
         
         ! Determine spheroid for call to UTMGEO
         ! NAD27 = 0 (Clark 1866); NAD83 and WGS-84 = 4 (GRS-80)
         SELECT CASE(Datum)
         CASE("NAD27")
            iSphere = 0
         CASE("NAD83")
            iSphere = 4
         END SELECT
        
         ! call UTMGEO to convert to lat/lon         
         CALL UTMGEO (333, CenterUTMZone, DummyZone, CenterUTME, 
     &                CenterUTMN, CenterLon, CenterLat, iSphere)

         ! The Center Latitude and Longitude are expressed in seconds on 
         ! return from UTMGEO; therefore the result needs to be divided by
         ! 3600. to obtain decimal degrees.  The longitude returned is 
         ! positive for North America; reverse to negative
         CenterLat = CenterLat/3600.0D0
         CenterLon = CenterLon/3600.0D0
      ELSE
         CenterLat = CenterLatIn
         CenterLon = CenterLonIn         
      END IF

C-----------------------------------------------------------------------
C     Determine if NAD shift is needed.  Shift is needed if datum of  
C     user's coordinates are inconsistent with the datum of the NLCD
C     file.  Binary state files are assumed to be NAD83, accepted 
C     datum for GeoTIFF files is NAD27, NAD83, WGS72, WGS84.  NAD83,
C     WGS72, and WGS83 are treated identically.  User coordinates will
C     be converted to match the datum of the NLCD file.
C-----------------------------------------------------------------------

      ! Set the KEY parameter needed by NADCON which indicates the direction
      ! of the conversion. 
      ! KEY = +1 indicates a transformation of NAD 27 to NAD 83 datums
      ! KEY = -1 indicates a psuedo-transformation of NAD 83 to NAD 27 datums
      
      ! NLCD File datum
      SELECT CASE(LCDatum)
      ! NLCD File = NAD27
      CASE("NAD27")
         ! User Coords
         SELECT CASE(Datum)
         ! User Coords = NAD83 => convert user coords to NAD27
         CASE("NAD83")
            NADShft = .true.
            Key = -1
         END SELECT
      
      !NLCD File = NAD83, WGS72, or WGS84
      CASE("NAD83","WGS72","WGS84")
         ! User Coords
         SELECT CASE(DATUM)
         ! User coords = NAD27 => convert user coords to NAD83
         CASE("NAD27")
            NADShft = .true.
            Key = +1
         END SELECT
      END SELECT
      
      ! Compute NAD shift and conversion
      SELECT CASE(NADShft)
      
      CASE(.true.)
         CALL NGRIDS (NODATA,.true.,0,NGPath)
	      IF( NODATA )THEN
	         WRITE(*,1250)
            WRITE(LogUnt,1250)
	         WRITE(OutUnt,1250)
	         STOP
	      ENDIF	
                    
 1250 FORMAT(/,' NAD GRID FILES MISSING (*.los & *.las)!',
     &       /,' AERSURFACE Processing Aborted!'/)
         
         ! Call NADCON 2.1 code to compute DATUM shifts and convert
         ! geographic coordinates between NAD27 and NAD83, and vice versa.
         
         ! NADCON(XPT,YPT,XPT2,YPT2,DLOS,DLAS,DLOM,DLAM,KEY)
         ! XPT, YPT: NAD27 coordinates input to NADCON when converting from NAD27 to NAD83
         !   However, when converting from NAD83 to NAD27, these coordinates are
         !   both input and output values.  Use to input NAD83 values and returned 
         !   NAD27 values converted from NAD83 (due to an interative guess/check 
         !   process in NADCON). Input as postive decimal degrees.
         ! XPT2, YPT2: NAD83 coordinates returned from NADCON when converting from NAD27
         !   to NAD83.  
         ! DLOS, DLAS: Lon/Lat shift in seconds (returned)
         ! DLOM, DLOS: Lon/Lat shift in meters (returned)
         ! KEY: +1 indicates a transformation of NAD 27 to NAD 83 datums
         !      -1 indicates a psuedo-transformation of NAD 83 to NAD 27 datums
c  ****************************************************************************************           
c           WARNING: Use temp variables for input and output coordinates.  NADCON 
c                    will reverse the signs of longitudinal values under certain
c                    conditions.
c  ****************************************************************************************         

         XPTIN =  CenterLon
         YPTIN =  CenterLat
                  
      
         SELECT CASE(Key)
         CASE(1)

            CALL NADCON(XPTIN,YPTIN,XPTOUT,YPTOUT,DLOS,DLAS,
     &                  DLOM,DLAM,KEY)
            ! set converted center coordinates
            CenterLon =   XPTOUT
            CenterLat =   YPTOUT
            
         CASE(-1)
            CALL NADCON(XPTIN,YPTIN,XPTOUT,YPTOUT,DLOS,DLAS,
     &                  DLOM,DLAM,KEY)
            ! set converted center coordinates
            CenterLon =   XPTIN
            CenterLat =   YPTIN
         END SELECT
         
c      CASE(.false.)
c         ! No conversion needed     
c         CenterLon = CenterLonIn
c         CenterLat = CenterLatIn     
         
      END SELECT
      
C     convert study center coordinates entered by user to albers coords.
      CALL LATLON2ALBERS (CenterLat,CenterLon,CntrAlbersX,CntrAlbersY)

c      print*, ProjOrigin, CentMeridian
c      print*, StdParallel1, StdParallel2
c      print*, Esquared, EarthRadius
c      print*, centerlonin, centerlatin
c      print*, centerlon, centerlat
c      print*, cntralbersx, cntralbersy


C-----------------------------------------------------------------------
C     Compute the column and row of the center of the study areay
C-----------------------------------------------------------------------
      StudyCenterCol =IDNINT((CntrAlbersX - DBLE(StateULX))/CellRes)+1_8
      StudyCenterRow =IDNINT((DBLE(StateULY) - CntrAlbersY)/CellRes)+1_8
      
      ! DEBUG TO GET LAT/LON FOR CENTER PIXEL
      
      CellX = dble(StateULX) + (dble(StudyCenterCol-1_8)*CellRes)
      CellY = dble(StateULY) - (dble(StudyCenterRow-1_8)*CellRes)
      CALL ALBERS2LATLON (CellX,CellY,CLat,CLon)
      
      ! END OF DEBUG
      
      WRITE(LogUnt, 1260) CenterLat, CenterLon, 
     &                    CntrAlbersX, CntrAlbersY,
     &                    StudyCenterRow, StudyCenterCol
 1260 FORMAT(/,' Based on the following input coordinates,',
     &         ' converted if needed to LATLON/NAD83: '/
     &         '    Latitude:  ', F12.6,/
     &         '    Longitude: ', F12.6,/
     &       /,' The study center is located at',/
     &         '    Albers projection X-coordinate: ',F10.0,/
     &         '    Albers projection Y-coordinate: ', F10.0,/
     &         '    Row in data file:', I8,/
     &         '    Column in data file:', I8)
     


C-----------------------------------------------------------------------
C     Compute the number and first and last rows and columns in study area 
C       for bowen ratio and albedo together and separately for Zo.
C     Derive coordinates for lower left corner of applicable portion 
C       of grids.
C-----------------------------------------------------------------------
      

     
      BaGridFrstRow = StudyCenterRow - (StudyRadiusBaAsCells)
      BaGridLastRow = StudyCenterRow + (StudyRadiusBaAsCells)
      BaGridFrstCol = StudyCenterCol - (StudyRadiusBaAsCells)
      BaGridLastCol = StudyCenterCol + (StudyRadiusBaAsCells)

      ZoGridFrstRow = StudyCenterRow - (StudyRadiusZ0AsCells)
      ZoGridLastRow = StudyCenterRow + (StudyRadiusZ0AsCells)
      ZoGridFrstCol = StudyCenterCol - (StudyRadiusZ0AsCells)
      ZoGridLastCol = StudyCenterCol + (StudyRadiusZ0AsCells)

      ZoGridNRows = ZoGridLastRow-ZoGridFrstRow+1_8
      ZoGridNCols = ZoGridLastCol-ZoGridFrstCol+1_8
      
      BaGridNRows = BaGridLastRow-BaGridFrstRow+1_8
      BaGridNCols = BaGridLastCol-BaGridFrstCol+1_8

      GridFrstRow = min(BaGridFrstRow,ZoGridFrstRow)
      GridLastRow = max(BaGridLastRow,ZoGridLastRow)
      GridFrstCol = min(BaGridFrstCol,ZoGridFrstCol)
      GridLastCol = max(BaGridLastCol,ZoGridLastCol)

C --- Allocate BaGrid and ZoGrid arrays
C     Add one to the cols to have a place to store the row coord (Albers Y)
      ALLOCATE(BaGrid(1_8:BaGridNRows,0_8:BaGridNCols+1_8))
      ALLOCATE(ZoGrid(1_8:ZoGridNRows,0_8:ZoGridNCols+1_8))
      
C --- Initialize arrays to 0 (kind=8) for missing, out-of-bounds, or invalid.     
      BaGrid = 0_8
      ZoGrid = 0_8


C --- Compute Albers lower left of bowen/albedo grid
      BaAlbersLLX = dble(StateULX) + dble(BaGridFrstCol-1_8)*CellRes -
     &                               (CellRes/2.0D0)
      BaAlbersLLY = dble(StateULY) - dble(BaGridLastRow-1_8)*CellRes -
     &                               (CellRes/2.0D0)

C --- Compute Albers lower left of roughness grid
      ZoAlbersLLX = dble(StateULX) + dble(ZoGridFrstCol-1_8)*CellRes -
     &                               (CellRes/2.0D0)
      ZoAlbersLLY = dble(StateULY) - dble(ZoGridLastRow-1_8)*CellRes -
     &                               (CellRes/2.0D0)


C-----------------------------------------------------------------------
C     Compute the amount of rotation needed to orient the study grid 
C     from grid north to true north.  This value will be applied to 
C     the flow angle computed for each cell using Albers coordinates 
C     when determining the user defined sector each cell resides in.  
C
C     Get lat/lon at top edge of study area from the center.  Compute
C     difference from center lat to edge lat.  (center - edge)  This is
C     amount needed to rotate grid to true-north.  Rotation is clockwise
C     in the west, counter clockwise in the east, and 0 on the -96.0 W
C     meridian
C-----------------------------------------------------------------------

C     compute difference in decimal degrees from grid-north to true-north

C     first, derive coordinates at the top (grid-north) edge of the study area
      EdgeAlbersX = CntrAlbersX
      EdgeAlbersY = CntrAlbersY + StudyRadiusMaxMeters

C     second, convert edge coordinates from Albers to lat/lon      
      CALL ALBERS2LATLON(EdgeAlbersX, EdgeAlbersY, EdgeLat, EdgeLon)

C     third, get Albers coordinates for due north of study center
      CALL LATLON2ALBERS (EdgeLat,CenterLon, EdgeAlbersX,EdgeAlbersY)

C     finally, calculate GridToTruDif angle from delta-X and delta-Y
C     from study center to due North at edge of study area
      GridToTruDif = DATAN2((EdgeAlbersX-CntrAlbersX),
     &                      (EdgeAlbersY-CntrAlbersY))
      GridToTruDif = GridToTruDif * Rad2Deg
      
      WRITE (LogUnt,600) GridToTruDif
  600 FORMAT(/,' Rotation of Albers grid to true North (deg): '
     &, F10.5,/)


C-----------------------------------------------------------------------
C     Confirm that the area of interest is within the boundaries of
C     the NLCD data
C-----------------------------------------------------------------------

      CALL CONFIRMBOUNDS(StudyRadiusMaxAsCells, StudyCenterCol, 
     &                   StudyCenterRow, StateCols, StateRows, 
     &                   OK2Proceed)

      IF( .NOT. OK2Proceed )THEN
         WRITE(*,700) CntrAlbersX, CntrAlbersY, StateULX, StateULY,
     &                  StateLRX, StateLRY
         WRITE(LogUnt,700) CntrAlbersX, CntrAlbersY, StateULX,
     &                   StateULY, StateLRX, StateLRY
         WRITE(OutUnt,700) CntrAlbersX, CntrAlbersY, StateULX,
     &                   StateULY, StateLRX, StateLRY
         STOP 
      ENDIF
      
  700 FORMAT( //
     &        ' The study area extends beyond the NLCD data file.',/
     &        ' Processing cannot complete.',//
     &        ' Study center coordinates (Albers X, Y): ',
     &         F10.0, ', ', F10.0/
     &        ' Upper left coordinates of data file (Albers X, Y): ',
     &         I9, ', ', I9/
     &        ' Lower right coordinates of data file (Albers X, Y): ',
     &         I9, ', ', I9 )



C-----------------------------------------------------------------------
C    Open NLCD file
C-----------------------------------------------------------------------

      OPEN( UNIT=InUnt, FILE=NLCDFile, FORM = 'UNFORMATTED', 
     &      STATUS='OLD', ACCESS='DIRECT', RECL=1, ERR=5000)
      WRITE(*,*)
      WRITE(*,*) ' The input file opened: ', NLCDFile


C ----------------------------------------------------------------------
C     Read data from NLCD file
C     NLCD file will be read differently depending on whether it is a
C     state binary, stripped tiff or tiled tiff file.
C     * State binary files - data begins at first byte in file;
C         treat as stripped tiff file with a single strip
C     * Stripped tiff files - data is organized in strips that are 
C         rowsPerStrip long and the width (in columns) of the image
C     * Tiled tiff files - data is organized into rectangular tiles that
C         tileLen (rows) long and tileWid (cols) wide
C ----------------------------------------------------------------------

          
C --- initialize counters (kind=8)
      BaGridRowCnt = 0_8
      ZoGridRowCnt = 0_8  

      select case(tiffType)
         
      ! bin files are processed as stripped tiff files with a single strip
      ! and an offset of 0 bytes
      case("strip")
           
         ! initialize tmprow = 0
         tmprow = 0_8 
                 
         ! loop over offsets array
         SOffsets: DO idos=1_8, size(dataOS,1,kind=8)
            
            SRowLoop: DO jrow = 1_8,rowsPerStrip
               
               ! increment row counter to keep track of current data row
               tmprow = tmprow+1_8
            
               ! initialize column to 0 for new row
               tmpcol = 0_8
               
               ! initialize column counts for bowen/albedo and roughness grids
               BaGridColCnt = 0_8
               ZoGridColCnt = 0_8

               ! cycle loop if first row has not been reached yet
               if (tmprow < GridFrstRow) then
                  cycle
               elseif (tmprow > GridLastRow .and. 
     &                                     (LC1992)) then 
                ! exit loop if current row exceeds last row to process, 
                ! unless NLCD Type not determined
                  exit
               end if
            
C ---          Compute cell Y coord - albers            
               CellY = dble(StateULY) - (dble(tmprow-1_8)*CellRes)

C ---          Increment counter and store values in applicable BaGrid and ZoGrid 
C              arrays for later output if within grid boundaries
               IF (tmprow >= BaGridFrstRow .and. 
     &             tmprow <= BaGridLastRow) THEN
                  BaGridRowCnt = tmprow - BaGridFrstRow + 1_8
                  BaGrid(BaGridRowCnt,0_8) = IDNINT(CellY)
               END IF
         
               IF (tmprow >= ZoGridFrstRow .and. 
     &             tmprow <= ZoGridLastRow) THEN
                  ZoGridRowCnt = tmprow - ZoGridFrstRow + 1_8
                  ZoGrid(ZoGridRowCnt,0_8) = IDNINT(CellY)
               END IF 
                  
               SColLoop: DO kcol = 1_8,StateCols
      
                  ! increment column counter to keep track of current data column
                  tmpcol = tmpcol+1_8
            
                  ! cycle loop if current column precedes first column to process
                  if (tmpcol < GridFrstCol .or. 
     &               (tmpcol > GridLastCol .and. 
     &                                      (LC1992)) ) then
                   ! cycle loop if current column exceeds last column to process, 
                   ! unless NLCD Type not determined
                     cycle
                  end if
               
C ---             Compute cell x coord - albers
                  CellX = dble(StateULX) + (dble(tmpcol-1_8)*CellRes)


C ---             Compute record number (implied 1 byte per record)
c                  = offset/bytesPerRecord + (no. samples from beginning of strip)*bytesPerSample/bytesPerRecord
                  thisrec = dataOS(idos) + 
     &                   ((((jrow-1_8)*StateCols) + tmpcol) * 
     &                      bytesPerSample)

C ---             initialize cell value to missing
                  CellLC = 0_8
                  
C ---             read land cover value
                  select case(bytesPerSample)
                  case(1)
                     READ(InUnt, rec=thisrec, ERR=5010) TempLC(1)
                     CellLC = INT(TempLC(1),kind=8)
                     
                  case(2)
                     READ(InUnt, rec=thisrec, ERR=5010) TempLC(1)
                     READ(InUnt, rec=thisrec+1_8, ERR=5010) TempLC(2)

                     if (byteswap) then 
                        tmpInt2 = Transfer((/TempLC(2),TempLC(1)/),
     &                                       tmpInt2)
                     else
                        tmpInt2 = Transfer((/TempLC(1),TempLC(2)/),
     &                                       tmpInt2)
                     end if
                     
                     CellLC = INT(tmpInt2,kind=8)

                  case(4)
                     READ(InUnt, rec=thisrec, ERR=5010) TempLC(1)
                     READ(InUnt, rec=thisrec+1_8, ERR=5010) TempLC(2)
                     READ(InUnt, rec=thisrec+2_8, ERR=5010) TempLC(3)
                     READ(InUnt, rec=thisrec+3_8, ERR=5010) TempLC(4)

                     if (byteswap) then 
                        CellLC = Transfer((/TempLC(4),TempLC(3),
     &                                  TempLC(2),TempLC(1)/),CellLC)
                     else
                        CellLC = Transfer((/TempLC(1),TempLC(2),
     &                                  TempLC(3),TempLC(4)/),CellLC)
                     end if
                     
                  end select

C ---             Determine if the LC value retrieved is valid and set
C                 NLCD year type flag if can be determined                  
                  call CheckLC(CellLC)

C ---             Increment counter and store values in applicable BaGrid and ZoGrid 
C                 arrays for later output if within grid boundaries
                  IF (tmprow >= BaGridFrstRow .and. 
     &                tmprow <= BaGridLastRow .and.
     &                tmpcol >= BaGridFrstCol .and. 
     &                tmpcol <= BaGridLastCol) THEN
                     BaGridColCnt = tmpcol - BaGridFrstCol + 1_8
                     BaGrid(BaGridRowCnt,BaGridColCnt) = CellLC 
                  END IF
                  IF (tmprow >= ZoGridFrstRow .and. 
     &                tmprow <= ZoGridLastRow .and.
     &                tmpcol >= ZoGridFrstCol .and.
     &                tmpcol <= ZoGridLastCol) THEN
                     ZoGridColCnt = tmpcol - ZoGridFrstCol + 1_8
                     ZoGrid(ZoGridRowCnt,ZoGridColCnt) = CellLC
                  END IF

               END DO SColLoop

            END DO SRowLoop
 
         END DO SOffsets 
                   
      case("tile")
             
         ! initialize tmprow = 0, tmpcol = 0, tileNum = 0
         tmprow = 0_8
         tmpcol = 0_8
         tileNum = 0_8
                 
         ! compute number of tiles across and down
         tilesAcross = (StateCols+tileWid-1_8)/tileWid
         tilesDown = (StateRows+tileLen-1_8)/tileLen
         
         ! loop over offsets array
         TOffsets: DO idos=1_8, size(dataOS,1,kind=8)         

            ! compute number of current tile, tile row, tile col
            ! first row in tile, first col in tile
            tileNum = tileNum+1_8
            tileRow = int((tileNum-1_8)/tilesAcross)+1_8
            tileCol = mod(tileNum-1_8,tilesAcross)+1_8
            tileFirstRow = ((tileRow-1_8)*tileLen)+1_8
            tileFirstCol = ((tileCol-1_8)*tileWid)+1_8
               
            ! compute current data row, decrement by 1 since incremented at 
            ! beginning of loop below
            tmprow = tileFirstRow-1_8
               
            ! loop over rows in tile
            TRowLoop: DO jrow=1_8, tileLen
            
               BaGridColCnt = 0_8
               ZoGridColCnt = 0_8
            
               ! increment row and compute column (decrement by 1 since incremented
               ! at beginning of loop below)
               tmprow = tmprow+1_8
               tmpcol = tileFirstCol-1_8

               ! cycle loop if current row precedes first row to process
               if (tmprow < GridFrstRow) then
                  cycle
               elseif (tmprow > GridLastRow .and. 
     &                                     (LC1992)) then 
                ! exit loop if current row exceeds last row to process, 
                ! unless NLCD Type not determined
                  exit
               end if

C ---          Compute cell Y coord - albers            
               CellY = dble(StateULY) - (dble(tmprow-1_8)*CellRes)

C ---          Increment counter and store values in applicable BaGrid and ZoGrid 
C              arrays for later output if within grid boundaries
               IF (tmprow >= BaGridFrstRow .and. 
     &             tmprow <= BaGridLastRow) THEN
                  BaGridRowCnt = tmprow - BaGridFrstRow + 1_8
                  BaGrid(BaGridRowCnt,0_8) = IDNINT(CellY)
               END IF
         
               IF (tmprow >= ZoGridFrstRow .and. 
     &             tmprow <= ZoGridLastRow) THEN
                  ZoGridRowCnt = tmprow - ZoGridFrstRow + 1_8
                  ZoGrid(ZoGridRowCnt,0_8) = IDNINT(CellY)
               END IF 
                  
               ! loop over columns in tile
               TColLoop: DO kcol=1_8, tileWid
               
                  ! increment column
                  tmpcol = tmpcol+1_8

C ---             Compute record number (implied 1 byte per record)
c                  = offset/bytesPerRecord + (no. samples from beginning of strip)*bytesPerSample/bytesPerRecord + 1
                  thisrec =  dataOS(idos) + 
     &                   ((((tmprow-tileFirstRow)*tileWid) + 
     &                      (tmpcol-tileFirstCol))*bytesPerSample) + 1_8

                  ! cycle loop if current column precedes first column to process
                  if (tmpcol < GridFrstCol .or. 
     &               (tmpcol > GridLastCol .and. 
     &                                      (LC1992)) ) then
                   ! cycle loop if current column exceeds last column to process, 
                   ! unless NLCD Type not determined
                     cycle
                  end if
               
C ---             Compute cell x coord - albers
                  CellX = dble(StateULX) + (dble(tmpcol-1_8)*CellRes)
                     
C ---             initialize cell value to missing
                  CellLC = 0_8
                  
C ---             read land cover value
                  select case(bytesPerSample)
                  case(1)
                     READ(InUnt, rec=thisrec, ERR=5010) TempLC(1)
                     CellLC = INT(TempLC(1),kind=8)
                     
                  case(2)
                     READ(InUnt, rec=thisrec, ERR=5010) TempLC(1)
                     READ(InUnt, rec=thisrec+1_8, ERR=5010) TempLC(2)

                     if (byteswap) then 
                        tmpInt2 = Transfer((/TempLC(2),TempLC(1)/),
     &                                       tmpInt2)
                     else
                        tmpInt2 = Transfer((/TempLC(1),TempLC(2)/),
     &                                       tmpInt2)
                     end if
                     
                     CellLC = INT(tmpInt2,kind=8)
                     
                  case(4)
                     READ(InUnt, rec=thisrec, ERR=5010) TempLC(1)
                     READ(InUnt, rec=thisrec+1_8, ERR=5010) TempLC(2)
                     READ(InUnt, rec=thisrec+2_8, ERR=5010) TempLC(3)
                     READ(InUnt, rec=thisrec+3_8, ERR=5010) TempLC(4)

                     if (byteswap) then 
                        CellLC = Transfer((/TempLC(4),TempLC(3),
     &                                  TempLC(2),TempLC(1)/),CellLC)
                     else
                        CellLC = Transfer((/TempLC(1),TempLC(2),
     &                                  TempLC(3),TempLC(4)/),CellLC)
                     end if
                     
                  end select

C ---             Determine if the LC value retrieved is valid and set
C                 NLCD year type flag if can be determined                  
                  call CheckLC(CellLC)

C ---             Increment counter and store values in applicable BaGrid and ZoGrid 
C                 arrays for later output if within grid boundaries
                  IF (tmprow >= BaGridFrstRow .and. 
     &                tmprow <= BaGridLastRow .and.
     &                tmpcol >= BaGridFrstCol .and. 
     &                tmpcol <= BaGridLastCol) THEN
                     BaGridColCnt = tmpcol - BaGridFrstCol + 1_8
                     BaGrid(BaGridRowCnt,BaGridColCnt) = CellLC
                  END IF
                  IF (tmprow >= ZoGridFrstRow .and. 
     &                tmprow <= ZoGridLastRow .and.
     &                tmpcol >= ZoGridFrstCol .and.
     &                tmpcol <= ZoGridLastCol) THEN
                     ZoGridColCnt = tmpcol - ZoGridFrstCol + 1_8
                     ZoGrid(ZoGridRowCnt,ZoGridColCnt) = CellLC
                  END IF

                END DO TColLoop ! tileWid, cols
            END DO TRowLoop ! tileLen, rows
         END DO TOffsets ! tile offsets         
      
      END SELECT

C----------------------------------------------------------------------- 
C --- Set NLCD type (1992)
C-----------------------------------------------------------------------
      
      if (LC1992) then
      
         nlcdType = 1992

      end if
 
C----------------------------------------------------------------------- 
C --- Load surface characteristic arrays based on the nlcd type
C-----------------------------------------------------------------------
      CALL InitLCSurfChar
      
      
C-----------------------------------------------------------------------
C    Consolidate seasonal sfc values into a single array based on user  
C    input for airport, sfc moisture, arid conditions, snow cover
C-----------------------------------------------------------------------

      CALL ConsolSfcVals
      
C----------------------------------------------------------------------- 
C --- Loop over grid array and process LC values - get surface char
C --- values and counts.  Use Bowen ratio/albedo grid since 
C --- it is known to be the largest and Zo is a subset.  ProcessLC
C --- assigns values and counts for both averages using one set of 
C --- values.
C-----------------------------------------------------------------------     
      
c      do jrow=BaGridFrstRow,BaGridLastRow
c         do icol=BaGridFrstCol,BaGridLastCol

      do jrow=1_8,size(BaGrid,1,kind=8)
         do icol=1_8,size(BaGrid,2,kind=8)-2_8
         
            ! compute row and column in file
            tmprow = BaGridFrstRow+(jrow-1_8)
            tmpcol = BaGridFrstCol+(icol-1_8)         
            
            ! compute CellX and CellY
            CellX = dble(StateULX) + (dble(tmpcol-1_8)*CellRes)
            CellY = dble(StateULY) - (dble(tmprow-1_8)*CellRes)
            
            ! process cell
            Call ProcessLC(CellX,CellY,BaGrid(jrow,icol))
            
         end do
      end do
      


C----------------------------------------------------------------------- 
C --- output bowen/albedo grid to log file
C-----------------------------------------------------------------------

      WRITE(LogUnt,'(/,a,/,2a,/)')
     &  ' Land cover grid for Albedo and Bowen Ratio.',
     &  ' These data are included in: ',trim(BaDomOFile)

      DO jrow = 1_8,BaGridNRows
         WRITE(LogUnt,1300) 
     &     (BaGrid(jrow,icol),icol=0_8,BaGridNCols)
      ENDDO
 1300 FORMAT(i11,'. : ',500(i2,1x))
 
      
C-----------------------------------------------------------------------
C     Create bowen/albedo grid file for import into Arc GIS
C-----------------------------------------------------------------------

      OPEN(UNIT=BaDomUnt, FILE=BaDomOFile, STATUS='UNKNOWN', ERR=5050)

      WRITE(BaDomUnt,'(a,t34,i10)') 'ncols',BaGridNCols
      WRITE(BaDomUnt,'(a,t34,i10)') 'nrows',BaGridNRows
      WRITE(BaDomUnt,'(a,t33,f11.1)') 'xllcorner',BaAlbersLLX
      WRITE(BaDomUnt,'(a,t33,f11.1)') 'yllcorner',BaAlbersLLY
      WRITE(BaDomUnt,'(a,t33,f11.2)') 'cellsize',CellRes
      WRITE(BaDomUnt,'(a,t39,a)') 'NODATA_value','0'

      DO jrow = 1_8,BaGridNRows
         WRITE(BaDomUnt,'(500(i2,1x))',ERR=5060)
     &       (BaGrid(jrow,icol),icol=1_8,BaGridNCols)
      ENDDO
      
      CLOSE(BaDomUnt)

C-----------------------------------------------------------------------
C     Output land cover counts for bowen/albedo to log file
C-----------------------------------------------------------------------

      WRITE(LogUnt,1301)
 1301 FORMAT(/' Land Cover Counts: Bowen Ratio and Albedo '/)
 
c --- output counts to log
      DO kdum = 0, NumLCTypes-1
         IF( LEN_TRIM(LCSurfChar(kdum) % ClassName) .NE. 0)THEN
            WRITE(LogUnt, 1302) kdum, Trim(LCSurfChar(kdum)%ClassName),
     &                 LandCoverBr(1,kdum)
         ENDIF
      END DO
 1302 FORMAT(1x,i3,1x,a40,': ',i6)
 
       WRITE(LogUnt,1303)
 1303 FORMAT(46('-'))
 

      WRITE(LogUnt,1304)'Total', 
     &      LandCovTotBr(1)
     
 1304 FORMAT(1x,a44,': ',i6)

 
C----------------------------------------------------------------------- 
C --- output roughness grid to log file
C-----------------------------------------------------------------------

      WRITE(LogUnt,'(/,a,/,2a,/)')
     &  ' Land cover grid for Surface Roughness.',
     &  ' These data are included in: ',trim(ZoDomOFile)

      DO jrow = 1_8,ZoGridNRows
         WRITE(LogUnt,1300) 
     &     (ZoGrid(jrow,icol),icol=0_8,ZoGridNCols)
      ENDDO
      
C-----------------------------------------------------------------------
C     Create roughness grid file for import into Arc GIS
C-----------------------------------------------------------------------

      OPEN(UNIT=ZoDomUnt, FILE=ZoDomOFile, STATUS='UNKNOWN', ERR=5070)

      WRITE(ZoDomUnt,'(a,t34,i10)') 'ncols',ZoGridNCols
      WRITE(ZoDomUnt,'(a,t34,i10)') 'nrows',ZoGridNRows
      WRITE(ZoDomUnt,'(a,t33,f11.1)') 'xllcorner',ZoAlbersLLX
      WRITE(ZoDomUnt,'(a,t33,f11.1)') 'yllcorner',ZoAlbersLLY
      WRITE(ZoDomUnt,'(a,t33,f11.2)') 'cellsize',CellRes
      WRITE(ZoDomUnt,'(a,t39,a)') 'NODATA_value','0'

      DO jrow = 1_8,ZoGridNRows
         WRITE(ZoDomUnt,'(500(i2,1x))',ERR=5080) 
     &       (ZoGrid(jrow,icol),icol=1_8,ZoGridNCols)
      ENDDO
      
      CLOSE(ZoDomUnt)

C-----------------------------------------------------------------------
C     Output land cover counts for roughness to log file
C-----------------------------------------------------------------------

      WRITE(LogUnt,1307)
 1307 FORMAT(/' Land Cover Counts: Surface Roughness ')
 
      
c     advance to next line of log file    
      WRITE(LogUnt,*)

      WRITE(LogUnt,1308) (isector,isector=1,NumSectors)
 1308 FORMAT(38x,' SECTOR:',5x,12(i2,4x))

      WRITE(LogUnt,1310) (StartDir(isector), isector=1,NumSectors)
 1310 FORMAT(27x,'Starting Direction:     ', 12(i3.3,3x))

      WRITE(LogUnt,1303)
      
c --- output counts to log
      DO kdum = 0, NumLCTypes-1
         IF( LEN_TRIM(LCSurfChar(kdum) % ClassName) .NE. 0)THEN
            WRITE(LogUnt, 1315) kdum, Trim(LCSurfChar(kdum)%ClassName),
     &                  (LandCoverZ0(isector,kdum), 
     &                   isector=1,NumSectors)
         ENDIF
      END DO
 1315 FORMAT(1x,i3,1x,a40,': ',12(i6))
 
c --- output total counts of cells per sector - to log
      WRITE(LogUnt,1303)
      WRITE(LogUnt,1316)'Total', 
     &      (LandCovTotZ0(isector),isector=1,NumSectors)
     
 1316 FORMAT(1x,a44,': ',12(i6))
 
c --- determine if count for Transitional class exceeds 10% in a sector (1992 NLCD Only)
c --- determine if count for missing (out of state boundary), 99, 
c     exceeds low bounds or high bounds in a sector; between low and high % 
c     generates a warning on screen and in log; >high boungs % terminates program
c     % must be calculated for B, alb separately from Z0
      WRITE(LogUnt, *) 

c     compute for % for Bowen ratio and albedo
      IF( LandCovTotBr(1) > 0 )THEN
c        Transition cells 
         IF (nlcdType == 1992) THEN        
            IF( REAL(LandCoverBr(1,TransCode))/
     &         REAL(LandCovTotBr(1))
     &         >  ExceedTrans )THEN
               WRITE(*, 1320) 
     &            (REAL(LandCoverBr(1,TransCode))
     &            /REAL(LandCovTotBr(1)))*100.,
     &            TransCode
               WRITE(LogUnt, 1320) 
     &            (REAL(LandCoverBr(1,TransCode))
     &            /REAL(LandCovTotBr(1)))*100.,
     &            TransCode
               WRITE(OutUnt, 1320) 
     &            (REAL(LandCoverBr(1,TransCode))
     &            /REAL(LandCovTotBr(1)))*100.,
     &            TransCode
            END IF
         END IF   
         
 1320 FORMAT(
     &  /,' WARNING: ',f5.1,'% of the cells extracted to compute ',
     &  /,' Bowen ratio and albedo are classified as "Transitional" 
     &   (',i2,').')
          
C        Missing data cells - outside of state boundary
         MissCountBa = LandCoverBr(1,MissCode)
         IF( REAL(MissCountBa)/
     &       REAL(LandCovTotBr(1)) >=  ExceedMissLow .and.
     &       REAL(MissCountBa)/
     &      REAL(LandCovTotBr(1)) <= ExceedMissHi )THEN
            WRITE(*, 1325)  
     &        (REAL(MissCountBa)
     &        /REAL(LandCovTotBr(1)))*100.
            WRITE(LogUnt, 1325) 
     &        (REAL(MissCountBa)
     &        /REAL(LandCovTotBr(1)))*100.
            WRITE(OutUnt, 1325) 
     &        (REAL(MissCountBa)
     &        /REAL(LandCovTotBr(1)))*100.
     
 1325 FORMAT( 
     &  /,' WARNING: The land cover values for ', f5.1, '% of',
     &  /,' the cells extracted to compute Bowen ratio and albedo',
     &  /,' are either missing, outside of the NLCD file boundary,',
     &  /,' or invalid.')         

         ELSEIF( REAL(MissCountBa)/
     &          REAL(LandCovTotBr(1)) > ExceedMissHi )THEN
            WRITE(*, 1326) 
     &        (REAL(MissCountBa)
     &        /REAL(LandCovTotBr(1)))*100.
            WRITE(LogUnt, 1326)  
     &        (REAL(MissCountBa)
     &        /REAL(LandCovTotBr(isector)))*100.
            WRITE(OutUnt, 1326)  
     &        (REAL(MissCountBa)
     &        /REAL(LandCovTotBr(isector)))*100.
            OK2Proceed = .FALSE.

 1326 FORMAT( 
     &  /,' ERROR: The land cover values for ', f5.1, '% of',
     &  /,' the cells extracted to compute Bowen ratio and albedo',
     &  /,' are either missing, outside of the NLCD file boundary,',
     &  /,' or invalid.')           
         ENDIF 
      ENDIF
         
      DO isector=1,NumSectors
c        compute for % for Z0
         IF( LandCovTotZ0(isector) > 0 )THEN
c           Transition cells  
            IF (nlcdType == 1992) THEN       
               IF( REAL(LandCoverZ0(isector,TransCode))/
     &            REAL(LandCovTotZ0(isector))
     &            >  ExceedTrans )THEN
                  WRITE(*, 1330) 
     &           (   REAL(LandCoverZ0(isector,TransCode))
     &               /REAL(LandCovTotZ0(isector)))*100.,
     &               isector, TransCode
                  WRITE(LogUnt, 1330) 
     &           (   REAL(LandCoverZ0(isector,TransCode))
     &               /REAL(LandCovTotZ0(isector)))*100.,
     &               isector, TransCode
                  WRITE(OutUnt, 1330) 
     &           (   REAL(LandCoverZ0(isector,TransCode))
     &               /REAL(LandCovTotZ0(isector)))*100.,
     &               isector, TransCode
               END IF
            END IF
            
 1330 FORMAT(/
     &  /,' WARNING: ',f5.1,'% of the cells extracted to compute ',
     &  /,' surface roughness in sector ',i2, ' are classified',
     &  /,' as "Transitional" (',i2,').')
            
C           Missing data cells - outside of state boundary
            MissCountZ0 = LandCoverZ0(isector,MissCode)
            IF( REAL(MissCountZ0)/
     &          REAL(LandCovTotZ0(isector)) >=  ExceedMissLow .and.
     &          REAL(MissCountZ0)/
     &          REAL(LandCovTotZ0(isector)) <= ExceedMissHi )THEN
               WRITE(*, 1335)  
     &           (REAL(MissCountZ0)
     &           /REAL(LandCovTotZ0(isector)))*100., isector
               WRITE(LogUnt, 1335) 
     &           (REAL(MissCountZ0)
     &           /REAL(LandCovTotZ0(isector)))*100., isector
               WRITE(OutUnt, 1335) 
     &           (REAL(MissCountZ0)
     &           /REAL(LandCovTotZ0(isector)))*100., isector
     
 1335 FORMAT( 
     &   /,' WARNING: The land cover values for ', f5.1, '% of',
     &   /,' the cells extracted to compute surface roughness',
     &   /,' in sector ',i2,' are either missing, outside of the',
     &   /,' NLCD file boundary, or invalid.')
     
            ELSEIF( REAL(MissCountZ0)/
     &             REAL(LandCovTotZ0(isector)) > ExceedMissHi )THEN
               WRITE(*, 1336) 
     &           (REAL(MissCountZ0)
     &           /REAL(LandCovTotZ0(isector)))*100., isector
               WRITE(LogUnt, 1336)  
     &           (REAL(MissCountZ0)
     &           /REAL(LandCovTotZ0(isector)))*100., isector
               WRITE(OutUnt, 1336)  
     &           (REAL(MissCountZ0)
     &           /REAL(LandCovTotZ0(isector)))*100., isector
               OK2Proceed = .FALSE.
               
 1336 FORMAT( 
     &   /,' ERROR: The land cover values for ', f5.1, '% of',
     &   /,' the cells extracted to compute surface roughness',
     &   /,' in sector ',i2,' are either missing, outside of the',
     &   /,' NLCD file boundary, or invalid.')
               
            ENDIF 
         ENDIF        
         
      ENDDO
      
c     abort processing if loop is on last sector    
      IF( .Not. OK2Proceed )THEN
         WRITE(*,1340)
         WRITE(LogUnt,1340)
         WRITE(OutUnt,1341)
         STOP
      ENDIF
 1340 FORMAT (/, ' AERSURFACE processing aborted!')
 1341 FORMAT (//,' AERSURFACE processing aborted!')
 
      CLOSE(inUnt)
 
      RETURN

 5000 WRITE(*,5005) trim(NLCDFile)
      WRITE(LogUnt,5005) trim(NLCDFile)
      WRITE(OutUnt,5005) trim(NLCDFile)
 5005 FORMAT(/,' There was a problem opening the NLCD file:'
     &       /,a,//,' Processing was aborted.')
      STOP

 5010 WRITE(*,5015) trim(NLCDFile)
      WRITE(LogUnt,5015) trim(NLCDFile)
      WRITE(OutUnt,5015) trim(NLCDFile)
 5015 FORMAT(/,' There was a problem reading the NLCD file:'
     &       /,a,//,' Processing was aborted.')
      STOP
      
 5050 WRITE(*,5055) trim(BaDomOFile)
      WRITE(LogUnt,5055) trim(BaDomOFile)
      WRITE(OutUnt,5055) trim(BaDomOFile)
 5055 FORMAT(
     &  /,' There was a problem opening the albedo/bowen grid file:'
     &  /,a,//,' Processing was aborted.')
      STOP 
     
 5060 WRITE(*,5065) trim(BaDomOFile)
      WRITE(LogUnt,5065) trim(BaDomOFile)
      WRITE(OutUnt,5065) trim(BaDomOFile)
 5065 FORMAT(
     &  /,' There was a problem writing to the albedo/bowen grid file:'
     &  /,a,//,' Processing was aborted.') 
      STOP

 5070 WRITE(*,5075) trim(ZoDomOFile)
      WRITE(LogUnt,5075) trim(ZoDomOFile)
      WRITE(OutUnt,5075) trim(ZoDomOFile)
 5075 FORMAT(
     &  /,' There was a problem opening the roughness grid file:'
     &  /,a,//,' Processing was aborted.') 
      STOP
     
 5080 WRITE(*,5085) trim(ZoDomOFile)
      WRITE(LogUnt,5085) trim(ZoDomOFile)
      WRITE(OutUnt,5085) trim(ZoDomOFile)
 5085 FORMAT(
     &  /,' There was a problem writing to the roughness grid file:'
     &  /,a,//,' Processing was aborted.')
      STOP

      
      END SUBROUTINE GetLCCounts
      

C ------------------------- Check Land Cover Category -----------------------       

      subroutine CheckLC(CellLC)
      
C --- Checks landcover category retrieved from a data cell and verifies if it is
C     valid against 1992 NLCD categories.  Flag is initialized to true, then set 
C     to false if an invalid value is encountered.  

      use ControlParams
      use LandCoverParams
      use FileUnits
      
      implicit none
      
      integer (kind=8), intent(inout)   :: CellLC  ! land cover category value
      
      integer                  :: iCat    ! loop counters
      
      ! flag to indicate LC value is valid
      logical                  :: LCValidFlg
      
C --- initialize variables
      LCValidFlg = .false.

C --- Check against 1992 categories
C --- 0 and negative values are treated as valid values and
C --- counted as missing or out-of-bounds (i.e., no data) 
C --- and are ignored here
      do iCat=1, size(LCCatsAll92)
         if (CellLC == LCCatsAll92(iCat) .or. CellLC <= 0) then
            LCValidFlg = .true.
         end if
      end do
      
C --- If any value > 0 is encountered that is not a 1992 LC category, 
C --- display message and abort
      if (.not. LCValidFlg) then
         write(*,8050) CellLC
         write(LogUnt,8050) CellLC
 8050 format(
     &    /," ERROR: A land cover category (", i2, ") was",
     &    /," encountered that conflicts with the 1992 NLCD",
     &    /," specifications. Processing aborted.")
         stop
      end if
          
      return
      
      end subroutine CheckLC
      
      
C ----------------- Byteswap 4-byte Integer (from integer) ------------------    

      subroutine byteswapi4_i(vIn,vOut)
c     Performs byteswap on 4-byte integer input as 4-byte integer
c     Accepts 1 4-byte integer and returns a swapped 4-byte integer.
      
      implicit none
      
      integer*4, intent(in)     :: vIn  ! 4-byte real value in (to be swapped)
      integer*4, intent(out)    :: vOut  ! 4-byte real value out (swapped value)
      
      
      integer*4              :: iTmp
      integer*4              :: iTmpSwp
      
      iTmp = transfer((vIn),iTmp)

      call mvbits(iTmp, 24, 8, iTmpSwp,  0)
      call mvbits(iTmp, 16, 8, iTmpSwp,  8)
      call mvbits(iTmp,  8, 8, iTmpSwp, 16)
      call mvbits(iTmp,  0, 8, iTmpSwp, 24)

      vOut = transfer(iTmpSwp, vOut) 
      
      end