      SUBROUTINE GetUserInput
      
C --- Prompt user for input values.  Echo input to output and log file.

      USE MyConstants
      USE AlbersProjectionData
      USE UserParams
      USE FileUnits
      USE ControlParams
      USE AvgParams

      IMPLICIT NONE
      
      CHARACTER (LEN=len(St3File)) :: tmpname  ! temporary variable for output filename

c     output string to indicate how months were assigned to seasons 
c     "Default" or "User-Specified"     
      CHARACTER (LEN=15)  :: AssignMnthStr   

      CHARACTER (LEN=10)  :: Buffer10        ! Character buffer used in error checking

      CHARACTER (LEN=4)   :: FileExtension   ! Store NLCD file extension for conversion to
                                             ! upper case to avoid case-sensitive issues 
                                             ! checking for '.tif' or '.bin' filetype

      INTEGER  :: ierr           ! track number of errors opening nlcd data file
                                 ! or specifying state postal code for .bin files;
                                 ! allow up to 4 tries before aborting to avoid
                                 ! compiler runtime error
      INTEGER  :: kdum           ! counters
      INTEGER  :: iarea, nsector ! counters
      INTEGER  :: nsectorchk     ! hold sector index for validation processing
      INTEGER  :: LocExt         ! location of file extension in file name
      INTEGER  :: ios            ! io status
      
      LOGICAL  :: FileExists = .False.  ! input file exist?
      LOGICAL  :: UsrMnthsOK = .True.   ! user assignment of months to seasons is OK
      LOGICAL  :: UsrMnthsRestart = .False.  ! flag to restart prompts to assign months to seasons
                                             ! User can enter -9 to restart assignments beginning with first season

C --- Initialize error counter, ierr
      ierr = 0

      FileExtension = '    '

C-----------------------------------------------------------------------
C     File Names
C        Enter the name of the input file
C        Check for the existence of the data file
C-----------------------------------------------------------------------
  500 WRITE(*,505)
  505 FORMAT(
     &  /,' Enter INPUT filename and extension (*.tif or *.bin)',
     &  /,' for land cover data file (include path if needed).',
     & //,' Enclose with quotes if path or filename contains spaces.',
     &  /,' Linux users include quotes to accommodate forward slash',
     &    ' in path.',
     &  /,' (132 character limit)',/)

      READ(*,*,iostat=ios,end=5499) NLCDFile

C --- tif and bin supported for state files, tif only for seamless

C --- Extract file extension and convert to upper case for checking
C     acceptable NLCD file type (.tif or .bin)
      FileExtension = NLCDFile(len_trim(NLCDFile)-3:len_trim(NLCDFile))
      CALL LWRUPR(FileExtension)

      IF (ios /= 0 .OR. len_trim(NLCDFile) < 5 .OR.
     &     (FileExtension /= '.TIF' .AND.
     &      FileExtension /= '.BIN')) THEN
         WRITE(*,510) TRIM(ADJUSTL(NLCDFile))
  510    FORMAT (
     &     /,' Invalid filename:',
     &     /,1x,a,
     &    //,' AERSURFACE can process both State and seamless',
     &     /,' GeoTIFF (.tif) files from the USGS, as well as',
     &     /,' State "binary" (.bin) files from the USGS.',
     &     /,' Your land cover data file should have either a',
     &     /,' .tif or .bin extension to indicate the file type.')
C ---    Increment error counter, ierr, and retry filename prompt,
C        up to 4 times
         ierr = ierr + 1
         IF( ierr .lt. 4 )then
            GO TO 500
         ELSE
            WRITE(*,100)
            WRITE(LogUnt,100)
            WRITE(OutUnt,100)
  100       FORMAT (/' ERROR processing NLCD filename. Aborting!')
            STOP
         ENDIF
      ENDIF

      INQUIRE( FILE=NLCDFile, EXIST=FileExists )
      
      IF( .NOT. FileExists )THEN
C ---    File does not exist; issue message and reprompt user (up to 4 times)
         WRITE(*,*)
         WRITE(*,*) ' The file ', Trim(NLCDFile), ' was not found'
C ---    Increment error counter, ierr, and retry filename prompt,
C        up to 5 times
         ierr = ierr + 1
         IF( ierr .lt. 5 )then
            GO TO 500
         ELSE
            WRITE(*,101)
            WRITE(LogUnt,101)
            WRITE(OutUnt,101)
  101       FORMAT (/' Specified NLCD file was not found. Aborting!')
            STOP
         ENDIF
      ELSE
C ---    File exists; now check for proper file type, .tif or .bin
         IF (FileExtension == '.TIF') THEN
            fileType = 'tif'
         ELSEIF (FileExtension == '.BIN') THEN
            fileType = 'bin'
         ELSE
C ---       Unsupported file type; issue message and reprompt user (up to 3 times)
            WRITE(*,*)
            WRITE(*,*) ' File type (',Trim(fileType),') not supported'
            WRITE(*,*) ' File must be GeoTIFF (.tif) or Binary (.bin)'
C ---       Increment error counter, ierr, and retry filename prompt,
C           up to 5 times
            ierr = ierr + 1
            IF( ierr .lt. 5 )then
               GO TO 500
            ELSE
               WRITE(*,102)
               WRITE(LogUnt,102)
               WRITE(OutUnt,102)
  102          FORMAT(/' Supported NLCD file type not found. Aborting!')
               STOP
            ENDIF
         ENDIF
      
      ENDIF

C --- Input file with proper file extension has been found;
C     Reset error counter, ierr, to 0
      ierr = 0

      IF( fileType == 'bin' )THEN
  200    WRITE(*,205)
  205    FORMAT(
     &      /,' Enter the 2-character postal code of the State data',
     &      /)

C ---    First check for character field > 2, to avoid interpreting
C        first 2 characters of LATLON or other inputs as valid code.
         READ(*,*,err=5498,end=5499) Buffer10
         IF( LEN_TRIM( Buffer10 ) .GT. 2 )THEN
            WRITE(*,*)
            WRITE(*,*) ' Postal Code does not appear to be valid'
            ierr = ierr + 1
            IF( ierr .lt. 4 )then
               GO TO 200
            ELSE
               WRITE(*,103)
               WRITE(LogUnt,103)
               WRITE(OutUnt,103)
  103          FORMAT(/' Valid Postal Code not found. Aborting!')
               STOP
            ENDIF
            GO TO 200
         ELSE
            READ(Buffer10,*,err=5498,end=5499) PostalCode
         ENDIF

         PostalCode = TRIM(PostalCode)
         CALL LWRUPR(PostalCode)

C ------ Get the array element from the STATEINFO array
         istate = 0
         DO iarea = 1,NAreas
            IF( STATEINFO(iarea) % StatePostalCode == PostalCode )THEN
               istate = iarea         ! NC = 33
               EXIT
            ENDIF
         ENDDO

         IF( istate == 0 )THEN
            WRITE(*,*)
            WRITE(*,*) ' Postal Code does not appear to be valid'
            ierr = ierr + 1
            IF( ierr .lt. 4 )then
               GO TO 200
            ELSE
               WRITE(*,103)
               WRITE(LogUnt,103)
               WRITE(OutUnt,103)
               STOP
            ENDIF
            GO TO 200
         ENDIF

         IF( PostalCode == 'AK'  .OR. PostalCode == 'HI' )THEN
            WRITE(*,*)
            WRITE(*,*) ' Data for Alaska and Hawaii are not available'
            WRITE(LogUnt,*)'Data for Alaska and Hawaii are not ',
     &                     'available'
            WRITE(OutUnt,*)'Data for Alaska and Hawaii are not ',
     &                     'available'
            STOP

         ELSEIF( PostalCode == 'CA' )THEN
C----       There are two areas (southern and northern) in California
C           East-West line from Watsonville (~20 km southeast of Santa
C           Cruz on Monteray Bay)

  300       WRITE(*, 305)
  305       FORMAT(
     &        /,' California data are divided into two regions ',
     &       //,'  Enter ''N'' if the study area is in the north',
     &        /,'  Enter ''S'' if the study area is in the south',/)
            READ(*,*,err=5498,end=5499) Region
            Region = TRIM(Region)
            CALL LWRUPR(Region)

            IF( Region == 'N' )THEN
               istate = 52
            ELSEIF( Region == 'S' )THEN
               istate = 53
            ELSE
               WRITE(*,425) Region
               GO TO 300
            ENDIF

         ELSEIF( PostalCode == 'TX' )THEN
C----       There are three areas (N, SW, and SE) in Texas
C           North: East-West line from south of Longview (in the east)
C                  to the junction of the western panhandle
C           Southwest/Southeast: North-South line at about Laredo

  400       WRITE(*,405)
  405       FORMAT(
     &        /,' Texas data are divided into three regions ',
     &        /,'  Enter ''N'' if the study area is in the north',
     &        /,'  Enter ''SW'' if the study area is in the southwest',
     &        /,'  Enter ''SE'' if the study area is in the southeast',
     &        /)
            READ( *,*,err=5498,end=5499 ) Region
            Region = TRIM(Region)
            CALL LWRUPR(Region)

            IF( Region == 'N' )THEN
               istate = 54
            ELSEIF( Region == 'SW' )THEN
               istate = 55
            ELSEIF( Region == 'SE' )THEN
               istate = 56
            ELSE
               WRITE(*,430) Region
               GO TO 400
            ENDIF
         ENDIF


      ENDIF


  425 FORMAT(' Invalid region for California ', A2, /)
  430 FORMAT(' Invalid region for Texas ', A2, /)



C-----------------------------------------------------------------------
C        Enter the output filename
C-----------------------------------------------------------------------
      
      CALL LWRUPR(CtlOFile)
      
  600 WRITE(*,605) TRIM(ADJUSTL(CtlOFile))
  605 FORMAT(
     &    /,' Enter name of OUTPUT file for the surface', 
     &    /,' characteristics formatted for AERMET Stage 3 input.',
     &   //,' ',a,' and ''*.LOG'' are not accepted output',
     &      ' filenames.',
     &    /,' Enclose with quotes if the path or filename contains',
     &      ' spaces.',
     &    /,' Linux users include quotes to accommodate forward slash',
     &      ' in path.',
     &    /,' (132 character limit)',/)
	  
      READ(*,*,err=5498,end=5499) St3File
      
      tmpname = TRIM(ADJUSTL(St3File))
      
      CALL LWRUPR(tmpname) 
      
      IF (TRIM(ADJUSTL(tmpname)) == TRIM(ADJUSTL(CtlOFile))) THEN
         WRITE(*,610) TRIM(ADJUSTL(St3File))
  610 FORMAT(
     &   /,' The filename entered is reserved.',/,1x,a)
         GO TO 600
      ENDIF

C---- The extension .LOG is not valid since we will create a log file
C     with the .log extension based on the output filename

      IF (LEN_TRIM(ADJUSTL(tmpname)) > 3) THEN
         IF ( tmpname(LEN_TRIM(ADJUSTL(tmpname))-3:
     &                LEN_TRIM(ADJUSTL(tmpname))) == '.LOG') THEN
            WRITE(*,610) TRIM(ADJUSTL(St3File))
            GO TO 600
         ENDIF
      ENDIF
      

C-----------------------------------------------------------------------
C     Specify the coordinate type for the center of the study area
C-----------------------------------------------------------------------

  700 WRITE(*,705)
  705 FORMAT(
     &   /,' Enter the type of coordinate used to define the center',
     &   /,' of the study area.',
     &  //,'   ''UTM'' for Universal Transverse Mercator coordinates',
     &   /,'   ''LATLON'' for Latitude and Longitude coordinates.',
     &   /,'   (Note: User responses are not case-sensitive.)',/)
      READ(*,*,err=5498,end=5499) CoordType
      CoordType = TRIM(CoordType)
      CALL LWRUPR(CoordType)


C-----------------------------------------------------------------------
C     Enter the location of the study area - the location must be in
C     expressed in meters for UTM coordinates and decimal degrees for
C     latitude and longitude
C-----------------------------------------------------------------------
      IF( CoordType == 'UTM' )THEN
  801    WRITE(*,805)
  805    FORMAT(/,' Enter the UTM Easting coordinate (in meters)',/)
         READ(*,*,iostat=ios,end=5499) CenterUTME
         
         IF( ios /= 0 )THEN
            WRITE(*,806)
  806       FORMAT( /,' Invalid entry.',/ )
            GOTO 801
         ENDIF

  809    WRITE(*,810)
  810    FORMAT(/,' Enter the UTM Northing coordinate (in meters)',/)
         READ(*,*,iostat=ios,end=5499) CenterUTMN
         
         IF( ios /= 0 )THEN
            WRITE(*,806)
            GOTO 809
         ENDIF

  814    WRITE(*,815)
  815    FORMAT(/,' Enter the UTM Zone (integer value)',/)
         READ(*,*,iostat=ios,end=5499) CenterUTMZone

         IF( ios /= 0 )THEN
            WRITE(*,806)
            GOTO 814
         ENDIF

  819    WRITE(*,820)
  820    FORMAT(
     &        /,' Enter the horizontal datum (''NAD83'' or ''NAD27'')',
     &        /)
         READ(*,*,err=5498,end=5499) Datum
         Datum = TRIM(Datum)
         CALL LWRUPR(Datum)
         
         IF( Datum /= 'NAD83' .AND. Datum /= 'NAD27' )THEN
            WRITE(*,806)
            GOTO 819
         ENDIF

      ELSEIF( CoordType == 'LATLON' )THEN
  824    WRITE(*,825)
  825    FORMAT(/,' Enter the Latitude (in decimal degrees)',/)
         READ(*,*,iostat=ios,end=5499) CenterLatIn
         
         IF( ios /= 0 )THEN
             WRITE(*,806)
             GOTO 824
         ENDIF

  829    WRITE(*,830)
  830    FORMAT(
     &        /,' Enter the Longitude (in decimal degrees)',/
     &          '   enter a negative number for west longitude,',/
     &          '   e.g., -75.63 for 75.63 degrees west',/)
         READ(*,*,iostat=ios,end=5499) CenterLonIn
         
         IF( ios /= 0 )THEN
             WRITE(*,806)
             GOTO 829
         ENDIF
         
C        prompt for datum
  831    WRITE(*,820)

         READ(*,*,err=5498,end=5499) Datum
         Datum = TRIM(Datum)
         CALL LWRUPR(Datum)
         
         IF( Datum /= 'NAD83' .AND. Datum /= 'NAD27' )THEN
            WRITE(*,806)
            GOTO 831
         ENDIF

      ELSE
         WRITE(*,835) CoordType
  835    FORMAT(/,' Invalid coordinate type entered: ', a6)
         GO TO 700
      ENDIF


C-----------------------------------------------------------------------
C     Define the radius of the study area for surface roughness, 
C     between 0.1 and 5.0 kilometers.  Recommended default value
C     is 1.0 kilometer.
C     
C     Study area for albedo and Bowen ratio is hardcoded as a
C     10km by 10km domain.
C-----------------------------------------------------------------------

  910 WRITE(*,912)
  912 FORMAT(
     &     /,' Enter the radius (in km) of the study area',
     &     /,' used for surface roughness (0.1 - 5.0 km).',
     &     /,' The recommended default value is 1 km.',/)
      READ(*,*,iostat=ios,end=5499) StudyRadiusZ0
      
      IF( ios /= 0 )THEN
         WRITE(*,806)
         GOTO 910
      ENDIF

	IF (StudyRadiusZ0 < 0.1D0) THEN
	   WRITE(*,*)
         WRITE(*,*) ' The Study Radius entered is less than 0.1 km'
         WRITE(*,*) ' Re-enter a value in the range of 0.1 - 5.0 km'
	   GOTO 910
	      
	ELSEIF (StudyRadiusZ0 > 5.0D0) THEN
	   WRITE(*,*)
         WRITE(*,*) ' The Study Radius entered is greater than 5.0 km'
         WRITE(*,*) ' Re-enter a value in the range of 0.1 - 5.0 km'
	   GOTO 910
	      
	ELSEIF (StudyRadiusZ0 /= 1.0D0) THEN
	   WRITE(*,*)
         WRITE(*,*) ' A non-default Study Radius has been entered.'
	     
      ENDIF
      
C-----------------------------------------------------------------------
C     Vary the surface roughness length by sector?
C     Moved from below - CRT 3/20/2007
C-----------------------------------------------------------------------

 1400 WRITE(*,1405)
 1405 FORMAT(
     &     /,' Do you want to define the surface roughness length',
     &     /,' for multiple sectors?',
     &    //,' Enter ''Y'' for Yes or ''N'' for No',/)
     
      READ(*,*,err=5498,end=5499) VaryBySector
      VaryBySector = TRIM(VaryBySector)
      CALL LWRUPR(VaryBySector)
      
      IF( VaryBySector /= 'Y' .AND. VaryBySector /= 'N' )THEN
         WRITE(*,*)
	      WRITE(*,*) ' Invalid Entry.'
         WRITE(*,*) ' Valid entries include ''Y'' and ''N'''
	      GO TO 1400
      ENDIF
	      


C-----------------------------------------------------------------------
C     Read in the number of sectors if user selects to vary by sector
C     Moved from below - CRT 3/20/2007
C-----------------------------------------------------------------------
      IF( VaryBySector == 'Y' )THEN
 1500    WRITE(*,1550)
         READ(*,*,iostat=ios,end=5499) NumSectors
         
         IF( ios /= 0 )THEN
            WRITE(*,806)
            GOTO 1500
         ENDIF

C ---    Varying the characteristics by sector means defining at least
C        two sectors, but no more than 12
         IF( NumSectors < 2 .OR. NumSectors > 12 )THEN
            WRITE(*,*)
            WRITE(*,*) ' Your response was to vary surface roughness'
            WRITE(*,*) ' by sector - enter a value from 2 to 12'
            GOTO 1500
         ENDIF
         
C ---    If user specified 12 sectors, assign start directions automatically
C        beginning with 0 degrees
         IF (NumSectors == 12) THEN
            StartDir(1) = 0
            DO nsector = 2,NumSectors
               StartDir(nsector) =  StartDir(nsector-1) + 30
            ENDDO
            
C        if less than 12 sectors, prompt for start directions
         ELSE           

            WRITE(*,1555)
            DO nsector = 1,NumSectors
 1510          WRITE(*,1560) nsector
               READ(*,*,iostat=ios,end=5499) StartDir(nsector)
            
               IF( ios /= 0 )THEN
                  WRITE(*,1567)
                  GOTO 1510
               ENDIF
            
C ----         Set 360 to 0
               IF (StartDir(nsector) == 360) THEN
                  StartDir(nsector) = 0
               ENDIF
            
C ----         Make sure the value is a valid direction (0 - 360)
               IF( StartDir(nsector) < 0 .OR. 
     &            StartDir(nsector) > 360 )THEN
                  WRITE(*,1570)
                  GOTO 1510
               ENDIF        

C ----         Make sure the value entered is unique; don't check it against itself
               IF( nsector > 1 )THEN
                  DO nsectorchk = nsector-1, 1, -1
                     IF( StartDir(nsector) == 
     &                   StartDir(nsectorchk) )THEN
                        WRITE(*,1580)
                        GOTO 1510
                     ENDIF
                  
C ---                Make sure the direction entered is greater than the previous
C                    entries by at least 30 degrees.               
                     IF( abs(StartDir(nsector)-StartDir(nsectorchk))
     &                  < 30 )THEN       
                        WRITE(*,1585)
                        GOTO 1510   
C                    Need to account for crossing 0 deg boundary               
                     ELSE
                        IF( StartDir(nsectorchk) >= 330 .AND.
     &                     (StartDir(nsector)+
     &                     (360-StartDir(nsectorchk)) < 30) )THEN       
                              WRITE(*,1585)
                              GOTO 1510   
                        ELSEIF( StartDir(nsector) >= 330 .AND.
     &                     (StartDir(nsectorchk)+
     &                     (360-StartDir(nsector)) < 30) )THEN       
                           WRITE(*,1585)
                           GOTO 1510   
                        ENDIF
                     ENDIF                    
                  
                  ENDDO
               ENDIF          
            
            ENDDO ! Prompt for start directions
            
         ENDIF ! Automatic assignments for 12 sectors or prompt for directions

         IF( NumSectors == 1 )THEN
            StartDir(nsector) = 0.0
         ENDIF

      ELSE
         WRITE(*,1565)
C        Single sector spanning all wind directions
         NumSectors = 1
      ENDIF

C---- Sort the sector boundaries into ascending order from smallest to
C     largest angle so the sector allocation (below) works properly.
      IF( NumSectors > 1 )THEN
         CALL STRAIGHTSORT(NumSectors, StartDir)
      ENDIF
      
      

C---- Format statements
 1550 FORMAT(
     &     /,' ***************************************************',
     &     /,' You can define up to 12 sectors with a minimum',
     &     /,' width of 30 degrees.  If you specify 12 sectors,'
     &     /,' the sector directions will be defined automatically',
     &     /,' every 30 degrees starting at 0 degrees North.',
     &     /,' ***************************************************',
     &    //,' Enter the number of sectors: ',/)
 1555 FORMAT(
     &     /,' ********************************************',
     &     /,' Enter sectors clockwise in ascending order',
     &     /,' in whole degrees (0 to 360 degrees). Decimal',
     &     /,' values will be truncated to whole degrees.',
     &     /,' ********************************************',/)
     
 1560 FORMAT( 
     &     /,' Enter the starting direction (in whole degrees)', 
     &     /,' for sector ',i2,/)
     
 1565 FORMAT(
     &     /,' A single sector for surface roughness length,',
     &     /,' was specified.',/)
     
 1567 FORMAT(
     &     /,' Invalid value. Please re-enter start direction in', 
     &     /,' whole degrees (0 to 360 degrees).',/)
 
 1570 FORMAT(
     &     /,' Invalid value.  Please enter a start direction in the',
     &     /,' valid range of 0 to 360 degrees.',/)
 
 1580 FORMAT(
     &     /,' The start direction entered has already been assigned',
     &     /,' to a sector.  Please enter a unique start direction.',/)
     
 1585 FORMAT(
     &    /,' The start direction entered must be at least 30',
     &    /,' degrees apart from all other start directions entered.',
     &    /,' Please re-enter the start direction.',/)
      

C-----------------------------------------------------------------------
C     Determine temporal resolution (annual, seasonal, monthly)
C-----------------------------------------------------------------------

 1600 WRITE(*,1605) WinNoSnowLbl,WinWSnowLbl,SpringLbl,SummerLbl,
     &              AutumnLbl
 1605 FORMAT(
     &  /,' Define the temporal resolution of surface characteristic', 
     &  /,' outputs (ANNUAL, SEASONAL or MONTHLY).',
     & //,' *********************************************************',
     &  /,' The reference surface characteristics used in AERSURFACE',
     &  /,' are stored as seasonal values for each land cover category',
     &  /,' based on the following seasonal categories:', 
     &  /,5(/,3x,a),
     & //,' SEASONAL surface characteristics output implies the',
     &  /,' following seasonal definitions based on AERMET:',
     & //,'   Dec, Jan, Feb for Winter (snow-covered or no snow)',
     &  /,'   Mar, Apr, May for Spring',
     &  /,'   Jun, Jul, Aug for Summer',
     &  /,'   Sep, Oct, Nov for Fall',
     & //,' Months can be reassigned to different seasonal categories',
     &  /,' for ANNUAL or MONTHLY output, including the option to',
     &  /,' specify continuous snow cover for one or more months.',
     &  /,' Months cannot be reassigned for SEASONAL output, and ',
     &  /,' Winter will assume either continuous snow cover or ',
     &  /,' no snow cover for all three months, depending on the ',
     &  /,' user response to the subsequent prompt.',
     &  /,' *********************************************************',
     & //,' Enter ''A'' for ANNUAL, ''S'' for SEASONAL, or', 
     &  /,'  ''M'' for MONTHLY:',/)
     
      READ(*,*,err=5498,end=5499) TemporalRes
      TemporalRes = TRIM(TemporalRes)
      CALL LWRUPR(TemporalRes)

      IF( TemporalRes /= 'A' .AND. 
     &    TemporalRes /= 'S' .AND.
     &    TemporalRes /= 'M' )THEN
         WRITE(*,1610)
 1610    FORMAT(/,' Entry for Temporal Resolution is not valid',/)
         GO TO 1600
      ELSEIF( TemporalRes == 'A' )THEN
         TempResStr = 'ANNUAL'
      ELSEIF( TemporalRes == 'S' )THEN
         TempResStr = 'SEASONAL'
      ELSEIF( TemporalRes == 'M' )THEN
         TempResStr = 'MONTHLY'
      ENDIF


C-----------------------------------------------------------------------
C     User defined season/month assignments
C     Prompt user for assignment if Temporal Resolution = MONTHLY or ANNUAL
C-----------------------------------------------------------------------

      IF (TemporalRes == 'A' .or. TemporalRes == 'M') THEN
      
 1700    WRITE(*,1705)
 1705    FORMAT(
     &     /,' Does the site experience continuous snow cover for at',
     &     /,' least one month?',
     &    //,' Enter ''Y'' for Yes or ''N'' for No ',/)

         READ(*,*,err=5498,end=5499) SnowChr

         SnowChr = TRIM(SnowChr)
         CALL LWRUPR(SnowChr)

	   IF( SnowChr /= 'Y' .AND. SnowChr /= 'N' )THEN
	      WRITE(*,*)
	      WRITE(*,*) ' Invalid Entry.'
            WRITE(*,*) ' Valid entries include ''Y'' and ''N'''
	      GO TO 1700
         ELSEIF( SnowChr == 'Y' )THEN
            Snow = .TRUE.
         ELSEIF( SnowChr == 'N' )THEN
            Snow = .FALSE.
         ENDIF    
      
         WRITE(*,1715)
 1715    FORMAT(
     &    /,' ******************************************************',
     &    /,' The default month-to-season associations in AERSURFACE',
     &    /,' will be as follows (where 1=Jan, 2=Feb, 3=Mar, etc.):')

C ---       display season labels accordingly based on continuous 
C           snow cover
         IF( Snow )THEN
            WRITE(*,1720) WinWSnowLbl
 1720       FORMAT(/,3x,a,': 12, 1, 2')
         ELSE
            WRITE(*,1723) WinNoSnowLbl
 1723       FORMAT(/,3x,a,': 12, 1, 2')
         ENDIF
         
         WRITE(*,1728) SpringLbl,SummerLbl,AutumnLbl
 1728    FORMAT(/,3x,a,': 3, 4, 5',//,
     &     3x,a,': 6, 7, 8 ',//,3x,a,': 9, 10, 11',/
     &     ' ******************************************************',/)
     
 1730    WRITE(*,1732)
 1732    FORMAT(/,
     &     ' Would you like to reassign the months to different ',
     &     'seasons?',//
     &     ' Enter ''Y'' to reassign or ''N'' ',
     &     'to use the default assignments.',/)

         READ(*,*,err=5498,end=5499) AssignMnthChr
         AssignMnthChr = TRIM(AssignMnthChr)
         CALL LWRUPR(AssignMnthChr)

 	      IF( AssignMnthChr /= 'Y' .AND. AssignMnthChr /= 'N' )THEN
 	         WRITE(*,*)
	         WRITE(*,*) ' Invalid Entry.'
            WRITE(*,*) ' Valid entries include ''Y'' and ''N'''
	         GO TO 1730
         ELSEIF( AssignMnthChr == 'Y' )THEN
            AssignMnth = .TRUE.
         ELSEIF( AssignMnthChr == 'N' )THEN
            AssignMnth = .FALSE.
         ENDIF

C ------ if user opted to reassign months
         IF( AssignMnth )THEN

            WRITE(*,1735) WinNoSnowLbl
 1735       FORMAT(
     &   /,' ****************************************************',
     &   /,' Assign months to seasons by entering a space or ',
     &   /,' comma-delimited list, e.g., to assign Nov, Dec, and ',
     &   /,' Jan, enter "11 12 1" (without the quotes).',
     &   /,' To skip a season enter "0" (no quotes).',
     &   /,' Enter "-9" to restart month-season assignments.',
     &  //,' You will be prompted for the following seasonal',
     &     ' categories:',
     &  //,4x,a,/)
     
            IF( Snow )THEN
               WRITE(*,'(4x,a,/)') trim(adjustl(WinWSnowLbl))
            ENDIF
            
            WRITE(*,1737) trim(adjustl(SpringLbl)),
     &                    trim(adjustl(SummerLbl)),
     &                    trim(adjustl(AutumnLbl))
 1737       FORMAT(
     &   4x,a,//,4x,a,//,4x,a,
     &   /,' ****************************************************')

C ---       prompt user for winter months without snow
 1740       WRITE(*,1745) WinNoSnowLbl
 1745       FORMAT(/,1x,a,':',/)
            READ(*,'(A60)',err=5498,end=5499) WinNoSnMnths
            
            UsrMnthsOK = .true.
            UsrMnthsRestart = .false.
            CALL PrsSeasons(WinNoSnMnths,1,UsrMnthsOK,UsrMnthsRestart)
            IF (UsrMnthsRestart .or. .NOT. UsrMnthsOK) THEN
               GOTO 1740
            ENDIF

C ---       if continous snow for at least one month, prompt
c           for winter with snow
            IF( Snow )THEN
 1746          WRITE(*,1745) WinWSnowLbl
               READ(*,'(A60)',err=5498,end=5499) WinWSnMnths
               CALL PrsSeasons(WinWSnMnths,2,UsrMnthsOK,UsrMnthsRestart)
               
               IF (UsrMnthsRestart) THEN
                  GOTO 1740
               ELSEIF (.NOT. UsrMnthsOK) THEN
                  GOTO 1746
               ENDIF
            ELSE
               WinWSnMnths = '0'
            ENDIF

C           prompt user for spring months
 1750       WRITE(*,1745) SpringLbl
            READ(*,'(A60)',err=5498,end=5499) SprgMnths

            CALL PrsSeasons(SprgMnths,3,UsrMnthsOK,UsrMnthsRestart)

            IF (UsrMnthsRestart) THEN
               GOTO 1740
            ELSEIF (.NOT. UsrMnthsOK) THEN
               GOTO 1750
            ENDIF

C           prompt user for summer months
 1760       WRITE(*,1745) SummerLbl
            READ(*,'(A60)',err=5498,end=5499) SumrMnths

            CALL PrsSeasons(SumrMnths,4,UsrMnthsOK,UsrMnthsRestart)

            IF (UsrMnthsRestart) THEN
               GOTO 1740
            ELSEIF (.NOT. UsrMnthsOK) THEN
               GOTO 1760
            ENDIF

C           prompt user for autumn months
 1770       WRITE(*,1745) AutumnLbl
            READ(*,'(A60)',err=5498,end=5499) AutmMnths

            CALL PrsSeasons(AutmMnths,5,UsrMnthsOK,UsrMnthsRestart)

            IF (UsrMnthsRestart) THEN
               GOTO 1740
            ELSEIF (.NOT. UsrMnthsOK) THEN
               GOTO 1770
            ENDIF

c ------    ensure all months are assigned, if not start over
            DO i=1,12  !12 months
               IF( SeasonMnths(i) == 0 )THEN
                  WRITE(*,1775)
 1775          FORMAT(/,
     &       ' Not all 12 months were assigned to a season.',/
     &       ' Please reassign months.')
C ---             reinitialize assignments to 0
                  SeasonMnths = 0
                  GOTO 1740
               ENDIF
            ENDDO
            
         ENDIF  ! If AssignMnth = True

      ELSEIF (TemporalRes == 'S') THEN

 1780    WRITE(*,1785)
 1785    FORMAT(
     &     /,' Does the site experience continuous snow cover for',
     &     /,' most of the winter?',
     &    //,'   If ''Yes'', then Dec, Jan, and Feb will assume',
     &     /,'   continuous snow cover.',
     &    //,'   If ''No'', then Dec, Jan, and Feb will assume',
     &     /,'   no snow cover.',
     &    //,'     Enter ''Y'' for Yes or ''N'' for No',/)

         READ(*,*,err=5498,end=5499) SnowChr

         SnowChr = TRIM(SnowChr)
         CALL LWRUPR(SnowChr)

	      IF( SnowChr /= 'Y' .AND. SnowChr /= 'N' )THEN
	         WRITE(*,*)
	         WRITE(*,*) ' Invalid Entry.'
            WRITE(*,*) ' Valid entries include ''Y'' and ''N'''
	         GO TO 1780
         ELSEIF( SnowChr == 'Y' )THEN
            Snow = .TRUE.
         ELSEIF( SnowChr == 'N' )THEN
            Snow = .FALSE.
         ENDIF          
      
      END IF 


C-----------------------------------------------------------------------
C     Initialize season/month assignments for default values
C     based on user input if months were not reassigned
C     SeasonMnths(1) refers to Jan, SeasonMnths(12) refers to Dec
C     value of 1=Winter w/o snow; 2=Winter w/ snow; 
C              3=Spring; 4=Summer; 5 = Autumn  
C-----------------------------------------------------------------------

      IF( .NOT. AssignMnth )THEN
         SeasonMnths(3)=3
         SeasonMnths(4)=3
         SeasonMnths(5)=3
         SeasonMnths(6)=4
         SeasonMnths(7)=4
         SeasonMnths(8)=4
         SeasonMnths(9)=5
         SeasonMnths(10)=5
         SeasonMnths(11)=5
         
C ---    special case for winter, continous snow or not
         IF( Snow )THEN
            SeasonMnths(1)=2
            SeasonMnths(2)=2
            SeasonMnths(12)=2
         ELSE
            SeasonMnths(1)=1
            SeasonMnths(2)=1
            SeasonMnths(12)=1
         ENDIF
         
C ------ set assignment strings for output
         IF (Snow) THEN
            WinNoSnMnths = '0'
            WinWSnMnths = '12 1 2'
         ELSE
            WinNoSnMnths = '12 1 2'
            WinWSnMnths = '0'
         ENDIF
         SprgMnths = '3 4 5'
         SumrMnths = '6 7 8'
         AutmMnths = '9 10 11'
      ENDIF


      
C-----------------------------------------------------------------------
C     Airport or non-airport
C-----------------------------------------------------------------------

 1000 WRITE(*,1005 )
 1005 FORMAT(
     &     /,' Is this site at an airport?',
     &    //,' Enter ''Y'' for Yes or ''N'' for No',/)
      READ(*,*,err=5498,end=5499) AirportChr

      AirportChr = TRIM(AirportChr)
      CALL LWRUPR(AirportChr)

	   IF( AirportChr /= 'Y' .AND. AirportChr /= 'N' )THEN
	      WRITE(*,*)
	      WRITE(*,*) ' Invalid Entry.'
         WRITE(*,*) ' Valid entries include ''Y'' and ''N'''
	      GO TO 1000
      ELSEIF( AirportChr == 'Y' )THEN
         Airport = .TRUE.
      ELSEIF( AirportChr == 'N' )THEN
         Airport = .FALSE.
      ENDIF


C-----------------------------------------------------------------------
C     Arid or Non-arid
C-----------------------------------------------------------------------
      
      IF (.not. Snow) THEN
      
 1100    WRITE(*,1105)
 1105 FORMAT( 
     &     /,' Is this site in an arid region?',
     &    //,' Enter ''Y'' for Yes or ''N'' for No',/)
         READ(*,*,err=5498,end=5499) AridChr

         AridChr = TRIM(AridChr)
         CALL LWRUPR(AridChr)

	      IF( AridChr /= 'Y' .AND. AridChr /= 'N' )THEN
	         WRITE(*,*)
	         WRITE(*,*) ' Invalid Entry.'
            WRITE(*,*) ' Valid entries include ''Y'' and ''N'''
	         GO TO 1100
         ELSEIF( AridChr == 'Y' )THEN
            Arid = .TRUE.
         ELSEIF( AridChr == 'N' )THEN
            Arid = .FALSE.
         ENDIF

      ELSE
         AridChr = 'N'
         Arid = .FALSE.
      ENDIF
      

C-----------------------------------------------------------------------
C     Surface Moisture: Average, Wet, Dry
C-----------------------------------------------------------------------

 1300 WRITE(*,1305)
 1305 FORMAT(
     &   /,' Characterize the surface moisture condition at the site',
     &   /,' relative to climatological normals, to be applied',
     &   /,' to all periods.',
     &  //,' Enter ''A'' for Average, ''W'' for Wet, or ',
     &     '''D'' for Dry',/)
      READ(*,*,err=5498,end=5499) Moisture

      Moisture = TRIM(Moisture)
      CALL LWRUPR(Moisture)

	   IF (Moisture /= 'A' .AND. Moisture /= 'W'
     &    .AND. Moisture /= 'D') THEN
         WRITE(*,*)
	      WRITE(*,*) ' Invalid Entry.  Valid entries include ''A'',',
     &              ' ''W'', and ''D'''
	      GO TO 1300
      ELSEIF (Moisture == 'A') THEN
         MoistStr = 'Average'
      ELSEIF (Moisture == 'W') THEN
         MoistStr = 'Wet'
      ELSEIF (Moisture == 'D') THEN
         MoistStr = 'Dry'
      ENDIF
          
      
C-----------------------------------------------------------------------
C     Create log filename
C
C     Open files
C        20 = NLCD data (InUnt,NLCDFile)
C        21 = AERMET Stage 3 (OutUnt,St3file)
C        22 = aersurface.dat (CtlOUnt,CtlOFile) (directed input from current run)
C        23 = log file (LogUnt,Logfile)
C
C     Echo inputs to log file, "control" file, and output file
C-----------------------------------------------------------------------
C

      IF(INDEX(St3File,'.',back=.true.) == 0 )THEN
         LogFile = TRIM(St3file) // ".log"
      ELSE
         LocExt = INDEX(St3File,'.',back=.true.)
         LogFile = St3file(1:LocExt-1)//".log"
      ENDIF


      OPEN(UNIT=OutUnt, FILE=St3file, STATUS='UNKNOWN', ERR=5199)
      WRITE(*,*) ' The output file opened: ', St3File

      OPEN(UNIT=LogUnt, FILE=Logfile, STATUS='UNKNOWN', ERR=5299)
      WRITE(*,*) ' The log file opened: ', Logfile
      
C --- Output Time and Version Info to Log file
C --- 2-mo, 3-day, 1-yr, 5-hr, 6-min, 7-sec
      WRITE(LogUnt,1805) 'Generated by AERSURFACE, dated ',
     &      Version,
     &      run_start(2), run_start(3), run_start(1),
     &      run_start(5), run_start(6)
 1805 FORMAT (1x,a,a10,18x,i2.2,'/',i2.2,'/',i4,
     &        1x,i2.2,':',i2.2,/)

C ----------------------------------------------------------------------
C     Open file to write user resposonses for directed input
C ----------------------------------------------------------------------
      OPEN(unit=CtlOUnt,file=trim(adjustl(CtlOFile)),
     &     status='unknown',err=5399)
     
C-----------------------------------------------------------------------
C     Write all the responses to the log file and control file
C-----------------------------------------------------------------------


c --- data file      
      WRITE(LogUnt,2050) TRIM(ADJUSTL(NLCDFile))
      IF (LEN_TRIM(ADJUSTL(NLCDFile)) <= 31 ) THEN
         WRITE(CtlOUnt,2051) TRIM(ADJUSTL(NLCDFile))
      ELSE
         WRITE(CtlOUnt,2052) TRIM(ADJUSTL(NLCDFile))
      ENDIF

c --- Postal Code if bin data
	   IF( fileType == 'bin' )THEN
      	WRITE(LogUnt,2005) TRIM(ADJUSTL(PostalCode))
      	WRITE(CtlOUnt,2006) TRIM(ADJUSTL(PostalCode))
        
c ---    Region if Postal Code = CA or TX
         IF( PostalCode == 'CA' .OR. PostalCode == 'TX' )THEN
            WRITE(LogUnt,2010) TRIM(ADJUSTL(Region))
            WRITE(CtlOUnt,2011) TRIM(ADJUSTL(Region))
         ENDIF 
     
      ENDIF

      
c --- output file with sfc chars for AERMET stg3 input
      WRITE(LogUnt,2055) TRIM(ADJUSTL(St3File))
      IF (LEN_TRIM(ADJUSTL(St3File)) <= 31 ) THEN
         WRITE(CtlOUnt,2056) TRIM(ADJUSTL(St3File))
      ELSE
         WRITE(CtlOUnt,2057) TRIM(ADJUSTL(St3File))
      ENDIF

c --- Coordinate Type      
      WRITE(LogUnt,2015) TRIM(ADJUSTL(CoordType))
      WRITE(CtlOUnt,2016) TRIM(ADJUSTL(CoordType))
      IF( CoordType == 'LATLON' )THEN
         WRITE(LogUnt,2020) CenterLatIn, CenterLonIn, 
     &         TRIM(ADJUSTL(Datum))
         WRITE(CtlOUnt,2021) CenterLatIn, CenterLonIn, 
     &         TRIM(ADJUSTL(Datum))
      ELSEIF( CoordType == 'UTM' )THEN
         WRITE(LogUnt,2025) CenterUTME, CenterUTMN, CenterUTMZone,
     &         TRIM(ADJUSTL(Datum))
         WRITE(CtlOUnt,2026) CenterUTME, CenterUTMN, CenterUTMZone,
     &         TRIM(ADJUSTL(Datum))
      ENDIF
      
c --- Study Radius for surface roughness
      WRITE(LogUnt,2031) StudyRadiusZ0
      WRITE(CtlOUnt,2032) StudyRadiusZ0

c --- Vary by Sector?
      WRITE(LogUnt,2035) TRIM(ADJUSTL(VaryBySector))
      WRITE(CtlOUnt,2036) TRIM(ADJUSTL(VaryBySector))

      IF( VaryBySector == 'Y' )THEN
         WRITE(LogUnt,2038) NumSectors,
     &              (StartDir(kdum),kdum=1,NumSectors)
         WRITE(CtlOUnt,2039) NumSectors
     
C        For 12 sectors, do not include start directions in 
c        control file since they are assigned automatically
         IF( NumSectors < 12 )THEN
            DO kdum=1,NumSectors
               WRITE(CtlOUnt,2040)StartDir(kdum),kdum
            ENDDO
         ENDIF
      ENDIF


C --- Temporal Resolution
      WRITE(LogUnt,2042) TempResStr
      WRITE(CtlOUnt,2043) TemporalRes

      IF (TemporalRes == 'A' .OR. TemporalRes == 'M') THEN
      
C ---    Snow Cover
         WRITE(LogUnt,2085) SnowChr
         WRITE(CtlOUnt,2086) SnowChr
         
         WRITE(LogUnt,2046) AssignMnthChr
         WRITE(CtlOUnt,2047) AssignMnthChr

         IF( AssignMnth )THEN
            
            WRITE(LogUnt,2048) WinNoSnowLbl,
     &         TRIM(ADJUSTL(WinNoSnMnths))

            WRITE(CtlOUnt,2049) TRIM(ADJUSTL(WinNoSnMnths)),
     &        WinNoSnowLbl

            IF( Snow ) THEN
               WRITE(LogUnt,2048) WinWSnowLbl, 
     &            TRIM(ADJUSTL(WinWSnMnths))
               WRITE(CtlOUnt,2049) TRIM(ADJUSTL(WinWSnMnths)),  
     &            WinWSnowLbl
            ENDIF
             
            WRITE(LogUnt,2048) SpringLbl, 
     &         TRIM(ADJUSTL(SprgMnths))
     
            WRITE(CtlOUnt,2049) TRIM(ADJUSTL(SprgMnths)),  
     &         SpringLbl

            WRITE(LogUnt,2048) SummerLbl, 
     &         TRIM(ADJUSTL(SumrMnths))
     
            WRITE(CtlOUnt,2049) TRIM(ADJUSTL(SumrMnths)),  
     &         SummerLbl
     
            WRITE(LogUnt,2048) AutumnLbl,
     &         TRIM(ADJUSTL(AutmMnths))
     
            WRITE(CtlOUnt,2049) TRIM(ADJUSTL(AutmMnths)),  
     &         AutumnLbl
         ENDIF

C     SEASONAL         
      ELSEIF( TemporalRes == 'S' )THEN
      
C ---    Snow Cover
         WRITE(LogUnt,2090) SnowChr
         WRITE(CtlOUnt,2091) SnowChr
             
      ENDIF  ! End of Temporal Resolution


c --- Airport?
      WRITE(LogUnt,2065) AirportChr
      WRITE(CtlOUnt,2066) AirportChr

c --- Arid? Ask only if there is not continuous snow cover
      IF (.NOT. Snow) THEN
         WRITE(LogUnt,2075) AridChr
         WRITE(CtlOUnt,2076) AridChr
      ENDIF


c --- Surface Moisture (Wet, Dry, Average)
      WRITE(LogUnt,2095) MoistStr
      WRITE(CtlOUnt,2096) Moisture

C --- Write header record and echo inputs to output file:
C
      WRITE(OutUnt,50) 
     &   '** Generated by AERSURFACE, dated ',trim(Version),
     &   '** Generated from "',trim(NLCDFile),'"'
   50 FORMAT (2a,/3a)
   

C --- set month/season assignment output string
      IF (AssignMnth) THEN
         AssignMnthStr = 'User-specified'
      ELSE
         AssignMnthStr = 'Default'
      ENDIF      
   
C --- output center coordinates
      IF( CoordType == 'LATLON' )THEN
         WRITE(OutUnt,60) CenterLatIn, CenterLonIn, Datum
         
   60    FORMAT('** Center Latitude (decimal degrees):  ', F12.6,/,
     &       '** Center Longitude (decimal degrees): ', F12.6,/,
     &       '** Datum: ', A)
     
      ELSEIF( CoordType == 'UTM' )THEN
         WRITE(OutUnt,65) CenterUTME, CenterUTMN, CenterUTMZone,
     &                      Datum
 65      FORMAT('** Center UTM Easting (meters):  ', F10.1,/,
     &       '** Center UTM Northing (meters): ', F10.1,/,
     &       '** UTM Zone: ', I3,'    Datum: ', A)
      ENDIF
     
C --- output study radius for surface roughness
      WRITE(OutUnt,71) StudyRadiusZ0   
  71  FORMAT('** Study radius (km) for surface', 
     &          ' roughness: ', F5.1) 
     
C --- output airport, snow cover,
      WRITE(OutUnt,73) AirportChr, SnowChr 
  73  FORMAT('** Airport? ',A,', Continuous snow cover? ',A) 

C --- sfc moisture, arid region
      WRITE(OutUnt,75) trim(adjustl(MoistStr)), AridChr
  75  FORMAT('** Surface moisture? ',A,
     &          ', Arid region? ',A)
     
C --- month/season assignmemts
      WRITE(OutUnt,80) trim(adjustl(AssignMnthStr))
  80  FORMAT('** Month/Season assignments? ',A)

      WRITE(OutUnt,82) 
     &    trim(adjustl(WinNoSnowLbl)),
     &    trim(adjustl(WinNoSnMnths)),
     &    trim(adjustl(WinWSnowLbl)),
     &    trim(adjustl(WinWSnMnths)),
     &    trim(adjustl(SpringLbl)),   
     &    trim(adjustl(SprgMnths)),  
     &    trim(adjustl(SummerLbl)),   
     &    trim(adjustl(SumrMnths)),   
     &    trim(adjustl(AutumnLbl)),   
     &    trim(adjustl(AutmMnths))
  82  FORMAT(5('** ',A,': ',A,/))


C --- Record Log File Name
      WRITE(LogUnt,2060) LogFile
      
C --- Write blank record to end of control file, to avoid runtime error 
C     if processing is aborted during redirected input
C      WRITE(CtlOUnt,*) ' '
      
c      CLOSE(CtlOUnt)
      

 2005 FORMAT(' State Postal Code: ',A)
 2006 FORMAT(A,T35,'** Postal Code')
 2010 FORMAT(' Region of State: ',A)
 2011 FORMAT(A,T35,'** Region of State')
 2015 FORMAT(' Type of Coordinates Entered: ', A)
 2016 FORMAT(A,T35,'** Coordinate type (UTM, LATLON)')
 2020 FORMAT('   Latitude (decimal degrees):  ', F12.6,/,
     &       '   Longitude (decimal degrees): ', F12.6,/,
     &       '   Datum: ', A)
 2021 FORMAT(F12.6,T35,'** Latitude', 
     &     /,F12.6,T35,'** Longitude',
     &     /,A,T35,'** Datum') 
     
 2025 FORMAT('   UTM Easting (meters):  ', F10.1,/,
     &       '   UTM Northing (meters): ', F10.1,/,
     &       '   UTM Zone: ', I3,/,
     &       '   Datum: ', A)
     
 2026 FORMAT(F10.1,T35,'** UTM-E', 
     &     /,F10.1,T35,'** UTM-N',
     &     /,I3,T35,'** UTM zone',
     &     /,A,T35,'** Datum')
     
 2031 FORMAT(' Study Radius for surface roughness (km): ',F7.1)
 2032 FORMAT(F7.1,T35,'** Study radius for surface roughness (km)')
 2035 FORMAT(' Is surface roughness varied by sector? ',A)
 2036 FORMAT(A,T35,'** Vary by sector? (Y/N)')
 2038 FORMAT('  # Sectors = ', I2,/,
     &       '    Sector beginning directions: ', 12I5)
 2039 FORMAT(I2,T35,'** Number of sectors')
 2040 FORMAT(I5,T35,'** Start of sector ',I2)
 
 2042 FORMAT(' Temporal resolution (ANNUAL, MONTHLY,',
     &       ' SEASONAL): ',A)
 2043 FORMAT(A,T35,'** Temporal resolution (A=ANNUAL, M=MONTHLY,',
     &       ' S=SEASONAL)')
 2046 FORMAT(' Reassign months to seasons? ',A)
 2047 FORMAT(A,T35,'** Reassign months to seasons? (Y/N)')
 2048 FORMAT(1x, A,': ',A)
 2049 FORMAT(A,T35,'** ',A)
 2050 FORMAT(' Land Cover input file opened: ',A)
 2051 FORMAT('"',A,'"',T35,'** Land use data file')
 2052 FORMAT('"',A,'"',2x,'** Land use data file')
 2055 FORMAT(' AERMET-formatted output file opened: ',A)
 2056 FORMAT('"',A,'"',T35,'** Output file with sfc values for',
     &       ' AERMET Stage 3')
 2057 FORMAT('"',A,'"',2x,'** Output file with sfc values for',
     &       ' AERMET Stage 3')
 2060 FORMAT(' Log file opened: ',A)
 2065 FORMAT(' Is site located at an airport? ',A)
 2066 FORMAT(A,T35,'** Airport? (Y/N)')
 2075 FORMAT(' Is site considered an arid region? ',A)
 2076 FORMAT(A,T35,'** Arid region? (Y/N)')
 2085 FORMAT(' Continuous snow cover for at least one month? ',A)
 2086 FORMAT(A,T35,'** Continuous snow cover at least one month? (Y/N)')
 2090 FORMAT(' Continuous snow cover for most of the winter? ',A)
 2091 FORMAT(A,T35,'** Continuous snow cover most of the winter? (Y/N)')
 2095 FORMAT(' Characterization of surface moisture at site: ',A)
 2096 FORMAT(A,T35,'** Surface Moisture (A=Average, W=Wet, D=Dry)')

      RETURN


 5199 WRITE(*,*) ' Error opening output file for AERMET ', St3File
      WRITE(LogUnt,*) ' Error opening output file for AERMET ', St3File
      WRITE(OutUnt,*) ' Error opening output file for AERMET ', St3File
      STOP

 5299 WRITE(*,*) ' Error opening log file ', LogFile
      WRITE(LogUnt,*) ' Error opening log file ', LogFile
      WRITE(OutUnt,*) ' Error opening log file ', LogFile
      STOP
      
 5399 WRITE(*,*) ' Error opening control file to record user input:'
      WRITE(*,*) ' ',CtlOFile
      WRITE(*,*) ' If you are trying to direct input from'
      WRITE(*,*) ' this file, rename the file and try again.'
      STOP

 5498 WRITE(*,*) ' Error reading control file.'
      WRITE(*,*) ' AERSURFACE processing aborted!'
      write(LogUnt,*) ' Error reading control file.'
      write(LogUnt,*) ' AERSURFACE processing aborted!'
      write(OutUnt,*) ' Error reading control file.'
      write(OutUnt,*) ' AERSURFACE processing aborted!'
      STOP

 5499 WRITE(*,*) ' Error reading control file; premature end-of-file.'
      WRITE(*,*) ' AERSURFACE processing aborted!'
      write(LogUnt,*) ' Error reading control file; ',
     &                 'premature end-of-file.'
      write(LogUnt,*) ' AERSURFACE processing aborted!'
      write(OutUnt,*) ' Error reading control file; ',
     &                 'premature end-of-file.'
      write(OutUnt,*) ' AERSURFACE processing aborted!'
      STOP

      RETURN
      
      END SUBROUTINE GetUserInput
      
      
      SUBROUTINE PrsSeasons(MnthStr,Season,UsrMnthsOK,UsrMnthsRestart)
C     parse string of months assigned to a season

      USE UserParams
      
      IMPLICIT NONE
      
      CHARACTER (LEN=60),INTENT(INOUT) :: MnthStr    ! string of months
      INTEGER,           INTENT(IN)  :: Season       ! 1-5
      INTEGER                        :: i            ! loop counter
      INTEGER                        :: lentoparse   ! length of the string to parse before '**'
      INTEGER                        :: numparsed    ! number of months parsed
      INTEGER                        :: ios          ! io status
      INTEGER                        :: IFC          ! no. of fields in MnthStr
      INTEGER                        :: IndxSeas(12) ! array of user entered month indices
      LOGICAL,           INTENT(OUT) :: UsrMnthsOK   ! indicates month assignments are OK
      LOGICAL,           INTENT(OUT) :: UsrMnthsRestart ! flag to restart months assignment entry (-9)

C --- initialize variables      
      UsrMnthsOK = .TRUE.
      UsrMnthsRestart = .FALSE.
      lentoparse = 0
      numparsed = 0
      IndxSeas = 0
      ifc = 0
      ios = 0
          
C --- set length of the portion of the string to parse (exclude comments)      
      lentoparse = INDEX(MnthStr,'**')-1
      
      IF( .NOT. lentoparse > 0 )THEN
         lentoparse = LEN_TRIM(MnthStr)
      ENDIF
          
      MnthStr = MnthStr(1:lentoparse)
      
C --- Define number of input "fields" in MnthStr, ifc
      CALL DEFINE(MnthStr,LENTOPARSE,IFC)
      
C --- Read month indices from input string      
      READ(MnthStr,*,iostat=ios) (IndxSeas(i),i=1,ifc)
      
      IF (ios /= 0) THEN
C ---    Read error
         WRITE( *,100)
  100    FORMAT( 
     &    /,' Invalid entry.',
     &    /,' Enter months as a space or',
     &      ' comma delimited list of integers (1-12).',
     &    /,' Enter 0 to skip the current season or -9 to restart the',
     &    /,' month-season assignments.',/)

      ELSE IF (IndxSeas(1) .EQ. -9) THEN
C ---    User entered "restart" code; set flag and reset array to 0's
         UsrMnthsOK = .FALSE.
         UsrMnthsRestart = .TRUE.
         SeasonMnths = 0
         WRITE(*,90)
   90    FORMAT( 
     &    /,' ''Restart'' code was entered for month-season',
     &      ' assignments.',/,' Reassign months.',/)
         RETURN
         
      ELSE IF (IndxSeas(1) .EQ. 0) THEN
C ---    Skip this month
         RETURN
         
      ELSE  
C ---    Loop through inputs and assign to Season      
         DO i = 1, ifc
            IF (IndxSeas(i) >= 1 .AND. IndxSeas(i) <= 12 )THEN
               IF (SeasonMnths(IndxSeas(i)) .EQ. 0) THEN
                  SeasonMnths(IndxSeas(i)) = Season
                  numparsed = numparsed + 1
               ELSE
                  WRITE(*,200) IndxSeas(i)
 200              FORMAT(
     &              /,' Month ',i2,' has already been assigned.',
     &              /,' Duplicate entries will be ignored.',/)
               ENDIF            
            ELSE
C ---          Invalid entry; month out-of-range
               WRITE(*,100)
               UsrMnthsOK = .FALSE.
               EXIT
            END IF            
         END DO
         
      END IF
      
C --- if no months were parsed, and user did not enter zero, return false
C     so user will be prompted again      
      IF( numparsed == 0 .AND. TRIM(ADJUSTL(MnthStr)) /= '0' )THEN
         UsrMnthsOK = .FALSE.
      ENDIF
      
      RETURN
      
      END SUBROUTINE PrsSeasons
      
      SUBROUTINE DEFINE(MnthStr,LENTOPARSE,IFC)
C***********************************************************************
C                 DEFINE Module of the AMS/EPA Regulatory Model - AERMOD
C
C        PURPOSE: Defines Location of Fields on Runstream Input Image
C
C        PROGRAMMER: Jeff Wang, Roger Brode
C
C        DATE:    March 2, 1992
C
C        INPUTS:  Input Runstream Card Image
C
C        OUTPUTS: Number of Fields on Card, IFC
C                 Beginning and Ending Columns of Fields, LOCB and LOCE
C
C        CALLED FROM:   SETUP
C***********************************************************************
C

C     Variable Declarations
      IMPLICIT NONE

      INTEGER :: I, IFC, LENTOPARSE
      LOGICAL :: INFLD
      CHARACTER (LEN=LENTOPARSE),INTENT(IN)  :: MnthStr    ! string of months

C     Initialize the Blank Line and In-field Status Indicators
      INFLD = .FALSE.

      IFC = 0

C     Loop through the Data Fields
      DO I = 1, LENTOPARSE

         IF (.NOT.INFLD .AND. 
     &            MnthStr(I:I).NE.' ' .AND. MnthStr(I:I).NE.',') THEN
C           Location is the Beginning of a Field
            INFLD = .TRUE.
C           Increment the Field Counter
            IFC = IFC + 1
         ELSE IF (INFLD .AND. 
     &           (MnthStr(I:I).EQ.' ' .OR. MnthStr(I:I).EQ.',')) THEN
C           Location is the End of a Field
C           Set Mark of Not In a field
            INFLD = .FALSE.
         END IF

      END DO

      RETURN
      END

