      SUBROUTINE ProcessLC(CellX,CellY,CellLC)
      
C --- Process land cover value extracted from NLCD file.  
C     Compute distance from center; determine sector when applicable;
C     accumulate counts and sums needed to compute averages.
C
C     August 2008, Clint R. Tillerson, MACTEC
C     (moved from GetLCCounts)
     
      
      USE AlbersProjectionData
      USE LandCoverParams
      USE MyConstants
      USE UserParams
      USE FileUnits
      USE ControlParams
      USE AvgParams

      IMPLICIT NONE

      ! input variables      
      DOUBLE PRECISION, intent(in)    :: CellX, CellY     ! albers coords of current cell
      INTEGER (KIND=8), intent(inout) :: CellLC           ! land cover value from cell
      
      ! used to compute angle of cell to center for sector 
      ! Lat and Lon difference between the center of the study
      ! area and the cell being processed
      DOUBLE PRECISION :: Xangle, Yangle
      
      ! Mathematical angle of the study center to the center
      ! of the cell of interest
      DOUBLE PRECISION :: CellAngle
      
      ! Flow vector angle of the study center to the center
      ! of the cell of interest (i.e., the head of the wind arrow)
      DOUBLE PRECISION :: FlowAngle  
      
      DOUBLE PRECISION :: Distance  ! distance to study center
      
      INTEGER :: WindSector !windsector reference
      
      INTEGER  :: kdum, mnum  ! loop counters

      ! Compute distance from center of current cell to study center 
      Distance =  DSQRT(ABS(CntrAlbersX-CellX)**2 +
     &                 (ABS(CntrAlbersY-CellY)**2))
     
      
C --- compute minimum distance limit from pixel center to study center 
C     to use when computing inverse distance weighted average for Zo
      MinD = pxlScalex*MinDFactr

C --- Any negative values are invalid.  Map to zero(0) if not caught prior.
      IF( CellLC < 0 ) CellLC = 0

C --- Derive cell sector based on user's sector definitions

      ! User did not define sectors, 1 sector by default
      IF( NumSectors == 1 )THEN

          WindSector = 1

      ! User defined sectors
      ELSE

         Xangle = CellX - CntrAlbersX
         Yangle = CellY - CntrAlbersY

         CellAngle = DATAN2(Yangle, Xangle)
         CellAngle = CellAngle * Rad2Deg
         
         CellAngle = CellAngle + GridToTruDif
         
         IF( CellAngle < 0.0D0 )THEN
            CellAngle = CellAngle + 360.0D0
         ENDIF
         
         ! compute flow angle
         FlowAngle = 90.0D0 - CellAngle
         
         IF( FlowAngle < 0.0D0 )THEN
            FlowAngle = FlowAngle + 360.0D0
         ENDIF
         
         WindSector = 0
         SectorLoop: DO kdum =1,NumSectors-1
            IF( FlowAngle >= StartDir(kdum) .AND.
     &          FlowAngle < StartDir(kdum+1) )Then
                WindSector = kdum
                EXIT SectorLoop
            ENDIF
         ENDDO SectorLoop    ! SectorLoop
         
         IF( WindSector == 0 )THEN
            WindSector = NumSectors
         ENDIF

      ENDIF                  ! # Sectors = 1

C --- keep a count of cells by land use category (ignore sector)
C     used for bowen ratio and albedo

      LandCoverBr(1,CellLC) = 
     &  LandCoverBr(1,CellLC) + 1
      LandCoverAlb(1,CellLC) = 
     &  LandCoverAlb(1,CellLC) + 1
     
C --- keep a total count of cells for bowen ratio and sector (ignore sector)
      LandCovTotBr(1) = 
     &  LandCovTotBr(1) + 1
      LandCovTotAlb(1) = 
     &  LandCovTotAlb(1) + 1
     

C --- keep a total count of cells for bowen and albedo excluding the 
C     missing and out of bounds (ignore sector)
      IF( CellLC /= MissCode )THEN
        LCCountBr(1) = LCCountBr(1)+ 1  
        LCCountAlb(1) = LCCountAlb(1)+ 1
      ENDIF


C --- keep a count for surface roughness
      IF (Distance <= StudyRadiusZ0Meters )THEN
         LandCoverZ0(WindSector,CellLC) = 
     &      LandCoverZ0(WindSector,CellLC) + 1    
         LandCovTotZ0(WindSector) = 
     &      LandCovTotZ0(WindSector) + 1 
     
C ---    keep a total count of cells by sector excluding the 
C        missing and out of bounds
         IF( CellLC /= MissCode )THEN                    
            LCCountZ0(WindSector) =
     &        LCCountZ0(WindSector)+1
         ENDIF
      ENDIF  


C --- for each month compute the geometric sums needed for average
C     bowen and albedo; ignore sector
      MnthLoop: DO mnum = 1, 12
      
         IF (CellLC /= MissCode) THEN

            BowenSum(1,mnum) = 
     &        BowenSum(1,mnum)+
     &        LOG(Bowen(CellLC,SeasonMnths(mnum)))

C           for Zo, limit the distance used for weighted average 
C           to MinD which is some fraction of the pixel resolution          
            IF( Distance <= StudyRadiusZ0Meters ) THEN
               IF (Distance < MinD) THEN
                  InvDSumZo(WindSector,mnum) = 
     &             InvDSumZo(WindSector,mnum)+
     &             (1.0/MinD**InvDPwr)
   
                  Z0Sum(WindSector,mnum) = 
     &              Z0Sum(WindSector,mnum)+
     &           ((1.0/MinD**InvDPwr)*
     &           LOG(Z0(CellLC,SeasonMnths(mnum))))
     
               ELSE
                  InvDSumZo(WindSector,mnum) = 
     &             InvDSumZo(WindSector,mnum)+
     &             (1.0/Distance**InvDPwr)
   
                  Z0Sum(WindSector,mnum) = 
     &              Z0Sum(WindSector,mnum)+
     &           ((1.0/Distance**InvDPwr)*
     &           LOG(Z0(CellLC,SeasonMnths(mnum))))
               
               END IF
            ENDIF 
            
         ENDIF
         
      END DO MnthLoop
      
      RETURN
               
      END SUBROUTINE